#
#
# Author: SEPP HOCHREITER
###############################################################################


setClass('Factorization',
         representation = representation(
           parameters = 'list',
           n = 'numeric',
           p1 = 'numeric',
           p2 = 'numeric',
           l = 'numeric',
           center = 'vector',
           scaleData = 'vector',
           X = 'matrix',
           L = 'matrix',
           Z = 'matrix',
           M = 'matrix',
           LZ = 'matrix',
           U = 'matrix',
           avini = 'vector',
           xavini = 'vector',
           ini = 'matrix',
           Psi = 'vector',
           lapla = 'matrix'))



setGeneric("summary")
setMethod("summary", "Factorization",
function(object, ...)
{
   cat("\nAn object of class",class(object),"\n\n")
   if (is(object) != "Factorization") {
        stop("Object object is not of the class Factorization. Stopped.")
    }
   cat("call:", deparse(object@parameters[[1]],0.75*getOption("width")),
        sep="\n\t")
    l<-object@l
    n<-object@n
    cat("\nNumber of rows: ",n, "\n")
    cat("\nNumber of columns: ",l, "\n")



    p1<-object@p1
    p2<-object@p2

    if (p1==p2) {
        cat("\nNumber of clusters: ",p2, "\n")
    } else {
        cat("\nNumber of row clusters: ",p1, "\n")
        cat("\nNumber of column clusters: ",p2, "\n")
    }


     cat("\n Information content of the clusters:\n")

    avini <- object@avini
    names(avini) <-paste("BC", c(1:p2,"sum"))


    print.default(round(avini,2), print.gap = 2)

     cat("\n\n")

     cat("\n Information content of the samples:\n")
    xavini <- object@xavini

    names(xavini) <-paste("Sample", c(1:l,"sum"))


    print.default(round(xavini,2), print.gap = 2)

     cat("\n Column clusters / Factors:\n")

    a <- summary.matrix(t(object@Z), digits = 3)
    colnames(a) <- paste("BC", c(1:p2))
    print(a)

     cat("\n")

     cat("\n Row clusters / Loadings:\n")
     a <- summary.matrix(object@L, digits = 3)
     colnames(a) <- paste("BC", c(1:p2))
   print(a)



})


setGeneric("show")
setMethod("show", "Factorization",
function(object)
{
   if (is(object) != "Factorization") {
        stop("Object object is not of the class Factorization. Stopped.")
    }
    l<-object@l
    n<-object@n
    p1<-object@p1
    p2<-object@p2

    which=c(1,2,3,4)

   if (length(which) > 1 && dev.interactive()) {
        devAskNewPage(ask = TRUE)
    }

    showf <- c(FALSE, FALSE,FALSE, FALSE)
    showf[which] <- TRUE


    if (showf[1]){
        names <- paste("BC", 1:p2)
        title <- "Information Content of Biclusters"

        barplot(object@avini[1:p2],names.arg=names,main=title)
    }


    if (showf[2]){

        names1 <- paste("Sample", 1:l)
        title1 <- "Information Content of Samples"

        barplot(object@xavini[1:l],names.arg=names1,main=title1)
    }


    if (showf[3]){


        title2 <- "Loadings of the Biclusters"

        boxplot(object@L,names=names,main=title2)
    }

    if (showf[4]){

        title3 <- "Factors of the Biclusters"

        boxplot(t(object@Z),names=names,main=title3)
    }

    devAskNewPage(ask = FALSE)


})

setGeneric("showSelected", function(object, ...) standardGeneric("showSelected"))
setMethod("showSelected", "Factorization", function(object, which=c(1,2,3,4))
{
   if (is(object) != "Factorization") {
        stop("Object object is not of the class Factorization. Stopped.")
    }
    l<-object@l
    n<-object@n
    p1<-object@p1
    p2<-object@p2



   if (length(which) > 1 && dev.interactive()) {
        devAskNewPage(ask = TRUE)
    }

    showf <- c(FALSE, FALSE,FALSE, FALSE)
    showf[which] <- TRUE


    if (showf[1]){
        names <- paste("BC", 1:p2)
        title <- "Information Content of Biclusters"

        barplot(object@avini[1:p2],names.arg=names,main=title)
    }


    if (showf[2]){

        names1 <- paste("Sample", 1:l)
        title1 <- "Information Content of Samples"

        barplot(object@xavini[1:l],names.arg=names1,main=title1)
    }


    if (showf[3]){

        names <- paste("BC", 1:p2)

        title2 <- "Loadings of the Biclusters"

        boxplot(object@L,names=names,main=title2)
    }

    if (showf[4]){

        names <- paste("BC", 1:p2)

        title3 <- "Factors of the Biclusters"

        boxplot(t(object@Z),names=names,main=title3)
    }

    devAskNewPage(ask = FALSE)


})






fabia <- function(X,p=5,alpha=0.1,cyc=500,spl=1.0,spz=1.0,random=1.0,center=2,norm=1,scale=0.0,lap=1.0){
	## X - data matrix
	## cyc - maximum number of cycles
        ## alpha - sparseness
        ## p - factors

        if (missing(X)) {
            stop("Data matrix X missing. Stopped.")
        }

        if (!is.matrix(X)) {
            X <- as.matrix(X)
        }

        if (!is.numeric(X)) {
            stop("Data matrix X must be numeric. Stopped.")
        }

        l=ncol(X)
        n=nrow(X)

        rownames(X) <- rownames(X, do.NULL = FALSE, prefix = "gene")
        colnames(X) <- colnames(X, do.NULL = FALSE, prefix = "sample")
        rowna <- rownames(X)
        colna <- colnames(X)



       cat("Running FABIA on ",n,"x",l," matrix with\n")
       cat("   Number of biclusters ---------------- p: ",p,"\n")
       cat("   Sparseness factor --------------- alpha: ",alpha,"\n")
       cat("   Number of iterations -------------- cyc: ",cyc,"\n")
       cat("   Loading prior parameter ----------- spl: ",spl,"\n")
       cat("   Factor prior parameter ------------ spz: ",spz,"\n")
       if (random>0) {
       cat("   Initialization loadings--------- random: ",random," = interval \n")
       } else {
       cat("   Initialization loadings--------- random: ",random," = SVD \n")
       }
       if (center>0) {
            if (center<2) {
       cat("   Centering ---------------------- center:  1  = mean\n")
            } else {
                if (center<3) {
       cat("   Centering ---------------------- center:  2  = median\n")
                } else {
       cat("   Centering ---------------------- center: ", center,"  = mode\n")
                }
            }
        } else {
       cat("   Centering ---------------------- center: ", center," = no centering\n")
        }
       if (norm>0) {
           if (norm > 1) {
       cat("   Scaling to variance one: --------- norm: ",norm," = Yes\n")
           } else {
       cat("   Quantile scaling (0.75-0.25): ---- norm: ",norm," = Yes\n")
           }} else {
       cat("   Scaling -------------------------- norm: ",norm," = No\n")
       }
       if (scale>0) {
       cat("   Scaling loadings per iteration -- scale: ",scale," = to ", scale,"\n")
       } else {
       cat("   Scaling loadings per iteration -- scale: ", scale," = No\n")
       }

       cat("   Constraint variational parameter -- lap: ", lap,"\n")




        eps <- as.double(1e-3)
        eps1 <- as.double(1e-10)
        init_lapla <- 1.0
        init_psi <- 0.2


        iin <-  1.0/l

        cent <- as.vector(rep(0.0,n))
        if (center>0) {
            if (center<2) {
                cent <- apply(X, 1, mean)
            } else {
                if (center<3) {
                    cent <- apply(X, 1, median)
                } else {
                    cent <- estimateMode(X)
                }
            }
        }

        X <- X - cent

        XX <- as.vector(rep(1,n))

        if (norm>0) {
            if (norm>1)
            {
                scaleData <-  1/sqrt(iin*apply(X,1,function(x) sum(x^2))+0.001*XX)
            }
            else
            {
                scaleData <-  1/sqrt(apply(X,1,function(x) {quantile(x,0.75) - quantile(x,0.25)})+0.001*XX)
            }
            X <- scaleData*X
        } else {
            scaleData <- as.vector(rep(1.0,n))
        }



         if (random>0) {
             L <- matrix(random*rnorm(n*p),nrow=n,ncol=p)
         } else {
             svd <- La.svd(X)
             L <- svd$u[,1:p]%*%diag(svd$d[1:p])
         }


        if (scale>0)
        {

            dL <- 1/sqrt( apply(L,1,function(x) sum(x^2))+0.001*as.vector(rep(1,n)))
            L <- (scale*dL)*L
        }


        lapla <-  init_lapla*matrix(1,nrow=l,ncol=p)

        Psi <- init_psi*XX



	cyc <- as.integer(cyc)
	alpha <- as.double(alpha)
	p <- as.integer(p)
	spz <- as.double(spz)
	scale <- as.double(scale)
	lap <- as.double(lap)



	res <- .Call("fabic", X,Psi,L,lapla,cyc ,alpha,eps,eps1,spl,spz,scale,lap,PACKAGE="fabia")




        # INI call for biclusters


        vz <- iin*apply(res$E_SX_n,1,function(x) sum(x^2))

        vz <- sqrt(vz+1e-10)

        ivz <- 1/vz

        if(length(ivz)==1) {
            nZ <- ivz*res$E_SX_n

            nL <- vz*res$L
        }
        else {
            nZ <- ivz*res$E_SX_n

            nL <- t(vz*t(res$L))
        }


        ini <- matrix(0,l,(p+1))
        avini <- as.vector(rep(0.0,(p+1)))
        xavini <- as.vector(rep(0.0,(l+1)))

        idp <- diag(p)
        ipsi <- 1/res$Psi

        for (j in 1:l){
                ppL <- crossprod(nL,ipsi*nL)
                mat <- idp + ppL/res$lapla[j,]
                ini[j,p+1] <- log(det(mat))

                for (i in 1:p){
                    ini[j,i] <- log(1+(1/res$lapla[j,i])*crossprod(nL[,i],ipsi*nL[,i]))
                }
            }
        for (i in 1:p){
            avini[i] <- sum(ini[,i])
        }
        for (i in 1:l){
            xavini[i] <- ini[i,p+1]
        }

        ss <- sum(ini[,p+1])
        xavini[l+1] <- ss
        avini[p+1] <- ss

        Lz=nL%*%nZ


        rownames(nL) <- rowna
        colnames(nL) <- colnames(nL, do.NULL = FALSE, prefix = "bicluster")
        clnames <- colnames(nL)

        rownames(nZ) <- clnames
        colnames(nZ) <- colna

        M <- diag(p)
        rownames(M) <- clnames
        colnames(M) <- clnames

        rownames(Lz) <- rowna
        colnames(Lz) <- colna

        U <- X-Lz

        rownames(U) <- rowna
        colnames(U) <- colna




        return(new('Factorization', parameters=list("fabia",cyc,alpha,spl,spz,p,NULL,NULL,random,scale,norm,center,lap),n=n,p1=p,p2=p,l=l,center=cent,scaleData=scaleData,X=X,L=nL,Z=nZ,M=M,LZ=Lz,U=U,avini=avini,xavini=xavini,ini=ini,Psi=res$Psi,lapla=res$lapla))


}

fabiap <- function(X,p=5,alpha=0.1,cyc=500,spl=1.0,spz=1.0,sL=0.6,sZ=0.6,random=1.0,center=2,norm=1,scale=0.0,lap=1.0){
	## X - data matrix
	## cyc - maximum number of cycles
        ## alpha - sparseness
        ## p - factors
        ## sL - final sparseness L
        ## sZ - final sparseness z

        if (missing(X)) {
            stop("Data matrix X missing. Stopped.")
        }

        if (!is.matrix(X)) {
            X <- as.matrix(X)
        }

        if (!is.numeric(X)) {
            stop("Data matrix X must be numeric. Stopped.")
        }

        l=ncol(X)
        n=nrow(X)

        rownames(X) <- rownames(X, do.NULL = FALSE, prefix = "gene")
        colnames(X) <- colnames(X, do.NULL = FALSE, prefix = "sample")
        rowna <- rownames(X)
        colna <- colnames(X)



       cat("Running FABIAP on ",n,"x",l," matrix with\n")
       cat("   Number of biclusters ---------------- p: ",p,"\n")
       cat("   Sparseness factor --------------- alpha: ",alpha,"\n")
       cat("   Number of iterations -------------- cyc: ",cyc,"\n")
       cat("   Loading prior parameter ----------- spl: ",spl,"\n")
       cat("   Factor prior parameter ------------ spz: ",spz,"\n")
       cat("   Loading sparseness projection ------ sL: ",sL,"\n")
       cat("   Factor sparseness projection ------- sZ: ",sZ,"\n")
       if (random>0) {
       cat("   Initialization loadings--------- random: ",random," = interval \n")
       } else {
       cat("   Initialization loadings--------- random: ",random," = SVD \n")
       }
       if (center>0) {
            if (center<2) {
       cat("   Centering ---------------------- center:  1  = mean\n")
            } else {
                if (center<3) {
       cat("   Centering ---------------------- center:  2  = median\n")
                } else {
       cat("   Centering ---------------------- center: ", center,"  = mode\n")
                }
            }
        } else {
       cat("   Centering ---------------------- center: ", center," = no centering\n")
        }
       if (norm>0) {
           if (norm > 1) {
       cat("   Scaling to variance one: --------- norm: ",norm," = Yes\n")
           } else {
       cat("   Quantile scaling (0.75-0.25): ---- norm: ",norm," = Yes\n")
           }} else {
       cat("   Scaling -------------------------- norm: ",norm," = No\n")
       }
       if (scale>0) {
       cat("   Scaling loadings per iteration -- scale: ",scale," = to ", scale,"\n")
       } else {
       cat("   Scaling loadings per iteration -- scale: ", scale," = No\n")
       }

       cat("   Constraint variational parameter -- lap: ", lap,"\n")



        eps <- as.double(1e-3)
        eps1 <- as.double(1e-10)
        init_lapla <- 1.0
        init_psi <- 0.2

        iin <-  1.0/l

        cent <- as.vector(rep(0.0,n))
        if (center>0) {
            if (center<2) {
                cent <- apply(X, 1, mean)
            } else {
                if (center<3) {
                    cent <- apply(X, 1, median)
                } else {
                    cent <- estimateMode(X)
                }
            }
        }

        X <- X - cent

        XX <- as.vector(rep(1,n))

        if (norm>0) {
            if (norm>1)
            {
                scaleData <-  1/sqrt(iin*apply(X,1,function(x) sum(x^2))+0.001*XX)
            }
            else
            {
                scaleData <-  1/sqrt(apply(X,1,function(x) {quantile(x,0.75) - quantile(x,0.25)})+0.001*XX)
            }
            X <- scaleData*X
        } else {
            scaleData <- as.vector(rep(1.0,n))
        }



        if (random>0) {
             L <- matrix(random*rnorm(n*p),nrow=n,ncol=p)
         } else {
             svd <- La.svd(X)
             L <- svd$u[,1:p]%*%diag(svd$d[1:p])
         }


        if (scale>0)
        {

            dL <- 1/sqrt( apply(L,1,function(x) sum(x^2))+0.001*as.vector(rep(1,n)))
            L <- (scale*dL)*L
        }


        lapla <-  init_lapla*matrix(1,nrow=l,ncol=p)

        Psi <- init_psi*XX


	cyc <- as.integer(cyc)
	alpha <- as.double(alpha)
	p <- as.integer(p)
	spz <- as.double(spz)
	scale <- as.double(scale)
	lap <- as.double(lap)


	res <- .Call("fabic", X,Psi,L,lapla,cyc,alpha,eps,eps1,spl,spz,scale,lap,PACKAGE="fabia")

        rL <- res$L
        n <- nrow(rL)
        gh1 <- sqrt(1.0*n)-(sqrt(1.0*n)-1.0)*sL
        for (i in 1:p)
        {
            #le <- sum(res$L[,i]^2)
            le <- 1.0
            rL[,i] <- projfunc(s=as.vector(res$L[,i]),k1=gh1,k2=le)
        }

        rz <- res$E_SX_n
        l <- ncol(rz)
        gh2 <- sqrt(1.0*l)-(sqrt(1.0*l)-1.0)*sZ
        for (i in 1:p)
        {
            #le <- sum(res$z[,i]^2)
            le <- 1.0
            rz[i,] <- projfunc(s=as.vector(res$E_SX_n[i,]),k1=2,k2=le)
        }


        # INI call for biclusters

        vz <- iin*apply(rz,1,function(x) sum(x^2))

        vz <- sqrt(vz+1e-10)

        ivz <- 1/vz

        if(length(ivz)==1) {
            nZ <- ivz*res$E_SX_n

            nL <- vz*res$L
        }
        else {
            nZ <- ivz*res$E_SX_n

            nL <- t(vz*t(res$L))
        }




        ini <- matrix(0,l,(p+1))
        avini <- as.vector(rep(0.0,(p+1)))
        xavini <- as.vector(rep(0.0,(l+1)))

        idp <- diag(p)
        ipsi <- 1/res$Psi

        for (j in 1:l){
                ppL <- crossprod(nL,ipsi*nL)
                mat <- idp + ppL/res$lapla[j,]
                ini[j,p+1] <- log(det(mat))

                for (i in 1:p){
                    ini[j,i] <- log(1+(1/res$lapla[j,i])*crossprod(nL[,i],ipsi*nL[,i]))
                }
            }
        for (i in 1:p){
            avini[i] <- sum(ini[,i])
        }
        for (i in 1:l){
            xavini[i] <- ini[i,p+1]
        }

        ss <- sum(ini[,p+1])
        xavini[l+1] <- ss
        avini[p+1] <- ss


        Lz=nL%*%nZ


        rownames(nL) <- rowna
        colnames(nL) <- colnames(nL, do.NULL = FALSE, prefix = "bicluster")
        clnames <- colnames(nL)

        rownames(nZ) <- clnames
        colnames(nZ) <- colna

        M <- diag(p)
        rownames(M) <- clnames
        colnames(M) <- clnames

        rownames(Lz) <- rowna
        colnames(Lz) <- colna

        U <- X-Lz

        rownames(U) <- rowna
        colnames(U) <- colna


    return(new('Factorization', parameters=list("fabiap",cyc,alpha,spl,spz,p,sL,sZ,random,scale,norm,center,lap),n=n,p1=p,p2=p,l=l,center=cent,scaleData=scaleData,X=X,L=nL,Z=nZ,M=M,LZ=Lz,U=U,avini=avini,xavini=xavini,ini=ini,Psi=res$Psi,lapla=res$lapla))

}

fabias <- function(X,p=5,alpha=0.6,cyc=500,spz=1.0,random=1.0,center=2,norm=1,lap=1.0){
	## X - data matrix
	## cyc - maximum number of cycles
        ## alpha - sparseness low value because enforced by projfunc
        ## p - factors

        if (missing(X)) {
            stop("Data matrix X missing. Stopped.")
        }

        if (!is.matrix(X)) {
            X <- as.matrix(X)
        }

        if (!is.numeric(X)) {
            stop("Data matrix X must be numeric. Stopped.")
        }


        l=ncol(X)
        n=nrow(X)

        rownames(X) <- rownames(X, do.NULL = FALSE, prefix = "gene")
        colnames(X) <- colnames(X, do.NULL = FALSE, prefix = "sample")
        rowna <- rownames(X)
        colna <- colnames(X)



       cat("Running FABIAS on ",n,"x",l," matrix with\n")
       cat("   Number of biclusters ---------------- p: ",p,"\n")
       cat("   Sparseness projection factor ---- alpha: ",alpha,"\n")
       cat("   Number of iterations -------------- cyc: ",cyc,"\n")
       cat("   Factor prior parameter ------------ spz: ",spz,"\n")
       if (random>0) {
       cat("   Initialization loadings--------- random: ",random," = interval \n")
       } else {
       cat("   Initialization loadings--------- random: ",random," = SVD \n")
       }
       if (center>0) {
            if (center<2) {
       cat("   Centering ---------------------- center:  1  = mean\n")
            } else {
                if (center<3) {
       cat("   Centering ---------------------- center:  2  = median\n")
                } else {
       cat("   Centering ---------------------- center: ", center,"  = mode\n")
                }
            }
        } else {
       cat("   Centering ---------------------- center: ", center," = no centering\n")
        }
       if (norm>0) {
           if (norm > 1) {
       cat("   Scaling to variance one: --------- norm: ",norm," = Yes\n")
           } else {
       cat("   Quantile scaling (0.75-0.25): ---- norm: ",norm," = Yes\n")
           }} else {
       cat("   Scaling -------------------------- norm: ",norm," = No\n")
       }

       cat("   Constraint variational parameter -- lap: ", lap,"\n")



       eps <- as.double(1e-3)
        eps1 <- as.double(1e-10)
        init_lapla <- 1.0
        init_psi <- 0.2

         iin <-  1.0/l

        cent <- as.vector(rep(0.0,n))
        if (center>0) {
            if (center<2) {
                cent <- apply(X, 1, mean)
            } else {
                if (center<3) {
                    cent <- apply(X, 1, median)
                } else {
                    cent <- estimateMode(X)
                }
            }
        }

        X <- X - cent

        XX <- as.vector(rep(1,n))

        if (norm>0) {
            if (norm>1)
            {
                scaleData <-  1/sqrt(iin*apply(X,1,function(x) sum(x^2))+0.001*XX)
            }
            else
            {
                scaleData <-  1/sqrt(apply(X,1,function(x) {quantile(x,0.75) - quantile(x,0.25)})+0.001*XX)
            }
            X <- scaleData*X
        } else {
            scaleData <- as.vector(rep(1.0,n))
        }



        if (random>0) {
             L <- matrix(random*rnorm(n*p),nrow=n,ncol=p)
         } else {
             svd <- La.svd(X)
             L <- svd$u[,1:p]%*%diag(svd$d[1:p])
         }




        lapla <-  init_lapla*matrix(1,nrow=l,ncol=p)

        Psi <- init_psi*XX


	cyc <- as.integer(cyc)
	alpha <- as.double(alpha)
	p <- as.integer(p)
	spz <- as.double(spz)
	lap <- as.double(lap)


	res <- .Call("fabics", X,Psi,L,lapla,cyc ,alpha,eps,spz,lap,PACKAGE="fabia")


       # INI call for biclusters

        vz <- iin*apply(res$E_SX_n,1,function(x) sum(x^2))

        vz <- sqrt(vz+1e-10)

        ivz <- 1/vz

        if(length(ivz)==1) {
            nZ <- ivz*res$E_SX_n

            nL <- vz*res$L
        }
        else {
            nZ <- ivz*res$E_SX_n

            nL <- t(vz*t(res$L))
        }





        ini <- matrix(0,l,(p+1))
        avini <- as.vector(rep(0.0,(p+1)))
        xavini <- as.vector(rep(0.0,(l+1)))

        idp <- diag(p)
        ipsi <- 1/res$Psi

        for (j in 1:l){
                ppL <- crossprod(nL,ipsi*nL)
                mat <- idp + ppL/res$lapla[j,]
                ini[j,p+1] <- log(det(mat))

                for (i in 1:p){
                    ini[j,i] <- log(1+(1/res$lapla[j,i])*crossprod(nL[,i],ipsi*nL[,i]))
                }
            }
        for (i in 1:p){
            avini[i] <- sum(ini[,i])
        }
        for (i in 1:l){
            xavini[i] <- ini[i,p+1]
        }

        ss <- sum(ini[,p+1])
        xavini[l+1] <- ss
        avini[p+1] <- ss

        Lz=nL%*%nZ


        rownames(nL) <- rowna
        colnames(nL) <- colnames(nL, do.NULL = FALSE, prefix = "bicluster")
        clnames <- colnames(nL)

        rownames(nZ) <- clnames
        colnames(nZ) <- colna

        M <- diag(p)
        rownames(M) <- clnames
        colnames(M) <- clnames

        rownames(Lz) <- rowna
        colnames(Lz) <- colna

        U <- X-Lz

        rownames(U) <- rowna
        colnames(U) <- colna

         return(new('Factorization', parameters=list("fabias",cyc,alpha,NULL,spz,p,NULL,NULL,random,scale,norm,center,lap),n=n,p1=p,p2=p,l=l,center=cent,scaleData=scaleData,X=X,L=nL,Z=nZ,M=M,LZ=Lz,U=U,avini=avini,xavini=xavini,ini=ini,Psi=res$Psi,lapla=res$lapla))


}


fabi <- function(X,p=5,alpha=0.1,cyc=500,spl=1.0,spz=1.0,center=2,norm=1){

        if (missing(X)) {
            stop("Data matrix X missing. Stopped.")
        }

        if (!is.matrix(X)) {
            X <- as.matrix(X)
        }

        if (!is.numeric(X)) {
            stop("Data matrix X must be numeric. Stopped.")
        }


        l=ncol(X)
        n=nrow(X)

        rownames(X) <- rownames(X, do.NULL = FALSE, prefix = "gene")
        colnames(X) <- colnames(X, do.NULL = FALSE, prefix = "sample")
        rowna <- rownames(X)
        colna <- colnames(X)



       cat("Running FABI on ",n,"x",l," matrix with\n")
       cat("   Number of biclusters ---------------- p: ",p,"\n")
       cat("   Sparseness factor --------------- alpha: ",alpha,"\n")
       cat("   Number of iterations -------------- cyc: ",cyc,"\n")
       cat("   Loading prior parameter ----------- spl: ",spl,"\n")
       cat("   Factor prior parameter ------------ spz: ",spz,"\n")
       if (center>0) {
            if (center<2) {
       cat("   Centering ---------------------- center:  1  = mean\n")
            } else {
                if (center<3) {
       cat("   Centering ---------------------- center:  2  = median\n")
                } else {
       cat("   Centering ---------------------- center: ", center,"  = mode\n")
                }
            }
        } else {
       cat("   Centering ---------------------- center: ", center," = no centering\n")
        }
       if (norm>0) {
           if (norm > 1) {
       cat("   Scaling to variance one: --------- norm: ",norm," = Yes\n")
           } else {
       cat("   Quantile scaling (0.75-0.25): ---- norm: ",norm," = Yes\n")
           }} else {
       cat("   Scaling -------------------------- norm: ",norm," = No\n")
       }

        eps <- as.double(1e-3)
        eps1 <- as.double(1e-10)
        init_lapla <- 1.0
        init_psi <- 0.2


        iin <-  1.0/l


        cent <- as.vector(rep(0.0,n))
        if (center>0) {
            if (center<2) {
                cent <- apply(X, 1, mean)
            } else {
                if (center<3) {
                    cent <- apply(X, 1, median)
                } else {
                    cent <- estimateMode(X)
                }
            }
        }

        X <- X - cent

        XX <- as.vector(rep(1,n))

        if (norm>0) {
            if (norm>1)
            {
                scaleData <-  1/sqrt(iin*apply(X,1,function(x) sum(x^2))+0.001*XX)
            }
            else
            {
                scaleData <-  1/sqrt(apply(X,1,function(x) {quantile(x,0.75) - quantile(x,0.25)})+0.001*XX)
            }
            X <- scaleData*X
        } else {
            scaleData <- as.vector(rep(1.0,n))
        }




       lapla <-  init_lapla*matrix(1,nrow=l,ncol=p)

        Psi <- init_psi*XX


        eps2 <- 1e-1



        kvect <- as.vector(rep(1,p))
        nvect <- as.vector(rep(1,n))
        nk_one <- matrix(1,n,p)
        nk_zero <- matrix(0,n,p)
        kk_zero <- matrix(0,p,p)
        kk_one <- diag(p)
	epsv<-eps1*kvect
	epsn<-eps*nvect
        E_SX_n <- matrix(0,p,l)
        E_SSXX_n <- list()

        L <- (0.5*XX^(.5))*nk_one
	for (i in 1:cyc){
                ipsi <- 1/Psi
		LPsi<-diag(ipsi)%*%L
		LPsiL<-crossprod(L,LPsi)
 		sum1<- nk_zero
		sum2<- eps2*kk_one
 		for (j in 1:l){
			tmp <- solve(LPsiL+diag(lapla[j,]))
			x_j <- as.vector(X[,j])
			e_sx_n <- as.vector(tcrossprod(tmp,LPsi)%*%x_j)
			#e_sx_n[which(e_sx_n<0)] <- 0
			e_ssxx_n <-  tmp + tcrossprod(e_sx_n,e_sx_n)
			sum1 <- sum1 +  tcrossprod(x_j,e_sx_n)
			sum2 <- sum2 + e_ssxx_n
			lapla[j,] <- (epsv+diag(e_ssxx_n))^(-spz)
                        lapla[j,which( lapla[j,]<1.0)] <- 1.0
		}
#                L <- (sum1 - alpha*sign(sum1))
#                L[which(abs(sum1)<alpha)] <- 0
                sll <- solve(sum2)
                L <- sum1%*%sll
                L <- L - alpha*Psi*sign(sum1)*abs(nk_one*eps+sum1)^{-spl}%*%sll
                L[which(abs(L)<abs(alpha*Psi*abs(nk_one*eps+sum1)^{-spl})%*%sll)] <- 0

                Psi <- epsn+abs(XX - diag(tcrossprod(L,sum1)/n))
                if (i %% 20==0) { print(i)}
	}

        ipsi <- 1/Psi
        LPsi<-diag(ipsi)%*%L
        LPsiL<-crossprod(L,LPsi)
        for (j in 1:l){
            tmp <- solve(LPsiL+diag(lapla[j,]))
            x_j <- as.vector(X[,j])
            e_sx_n <- as.vector(tcrossprod(tmp,LPsi)%*%x_j)
            E_SX_n[,j] <- e_sx_n
            E_SSXX_n[[j]] <-  tmp + tcrossprod(e_sx_n,e_sx_n)
        }

        # INI call for biclusters

        vz <- iin*apply(E_SX_n,1,function(x) sum(x^2))

        vz <- sqrt(vz+1e-10)

        ivz <- 1/vz

        if(length(ivz)==1) {
            nZ <- ivz*E_SX_n

            nL <- vz*L
        }
        else {
            nZ <- ivz*E_SX_n

            nL <- t(vz*t(L))
        }



        ini <- matrix(0,l,(p+1))
        avini <- as.vector(rep(0.0,(p+1)))
        xavini <- as.vector(rep(0.0,(l+1)))

        idp <- diag(p)
        ipsi <- 1/Psi

        for (j in 1:l){
                ppL <- crossprod(nL,ipsi*nL)
                mat <- idp + ppL/lapla[j,]
                ini[j,p+1] <- log(det(mat))

                for (i in 1:p){
                    ini[j,i] <- log(1+(1/lapla[j,i])*crossprod(nL[,i],ipsi*nL[,i]))
                }
            }
        for (i in 1:p){
            avini[i] <- sum(ini[,i])
        }
        for (i in 1:l){
            xavini[i] <- ini[i,p+1]
        }

        ss <- sum(ini[,p+1])
        xavini[l+1] <- ss
        avini[p+1] <- ss


        Lz=nL%*%nZ


        rownames(nL) <- rowna
        colnames(nL) <- colnames(nL, do.NULL = FALSE, prefix = "bicluster")
        clnames <- colnames(nL)

        rownames(nZ) <- clnames
        colnames(nZ) <- colna

        M <- diag(p)
        rownames(M) <- clnames
        colnames(M) <- clnames

        rownames(Lz) <- rowna
        colnames(Lz) <- colna

        U <- X-Lz

        rownames(U) <- rowna
        colnames(U) <- colna

      return(new('Factorization', parameters=list("fabi",cyc,alpha,spl,spz,p,NULL,NULL,NULL,scale,norm,center,NULL),n=n,p1=p,p2=p,l=l,center=cent,scaleData=scaleData,X=X,L=nL,Z=nZ,M=M,LZ=Lz,U=U,avini=avini,xavini=xavini,ini=ini,Psi=Psi,lapla=lapla))

}


fabiasp <- function(X,p=5,alpha=0.6,cyc=500,spz=1.0,center=2,norm=1){


         if (missing(X)) {
            stop("Data matrix X missing. Stopped.")
        }

        if (!is.matrix(X)) {
            X <- as.matrix(X)
        }

        if (!is.numeric(X)) {
            stop("Data matrix X must be numeric. Stopped.")
        }


        l=ncol(X)
        n=nrow(X)

        rownames(X) <- rownames(X, do.NULL = FALSE, prefix = "gene")
        colnames(X) <- colnames(X, do.NULL = FALSE, prefix = "sample")
        rowna <- rownames(X)
        colna <- colnames(X)



       cat("Running FABIASP on ",n,"x",l," matrix with\n")
       cat("   Number of biclusters ---------------- p: ",p,"\n")
       cat("   Sparseness projection factor ---- alpha: ",alpha,"\n")
       cat("   Number of iterations -------------- cyc: ",cyc,"\n")
       cat("   Factor prior parameter ------------ spz: ",spz,"\n")
       if (center>0) {
            if (center<2) {
       cat("   Centering ---------------------- center:  1  = mean\n")
            } else {
                if (center<3) {
       cat("   Centering ---------------------- center:  2  = median\n")
                } else {
       cat("   Centering ---------------------- center: ", center,"  = mode\n")
                }
            }
        } else {
       cat("   Centering ---------------------- center: ", center," = no centering\n")
        }
       if (norm>0) {
           if (norm > 1) {
       cat("   Scaling to variance one: --------- norm: ",norm," = Yes\n")
           } else {
       cat("   Quantile scaling (0.75-0.25): ---- norm: ",norm," = Yes\n")
           }} else {
       cat("   Scaling -------------------------- norm: ",norm," = No\n")
       }





        eps <- as.double(1e-3)
        eps1 <- as.double(1e-10)
        init_lapla <- 1.0
        init_psi <- 0.2


         iin <-  1.0/l

        cent <- as.vector(rep(0.0,n))
        if (center>0) {
            if (center<2) {
                cent <- apply(X, 1, mean)
            } else {
                if (center<3) {
                    cent <- apply(X, 1, median)
                } else {
                    cent <- estimateMode(X)
                }
            }
        }

        X <- X - cent

         XX <- as.vector(rep(1,n))

        if (norm>0) {
            if (norm>1)
            {
                scaleData <-  1/sqrt(iin*apply(X,1,function(x) sum(x^2))+0.001*XX)
            }
            else
            {
                scaleData <-  1/sqrt(apply(X,1,function(x) {quantile(x,0.75) - quantile(x,0.25)})+0.001*XX)
            }
            X <- scaleData*X
        } else {
            scaleData <- as.vector(rep(1.0,n))
        }



        lapla <-  init_lapla*matrix(1,nrow=l,ncol=p)

        Psi <- init_psi*XX


        eps2 <- 1e-1



        L1a <- sqrt(n)-(sqrt(n)-1)*alpha



        kvect <- as.vector(rep(1,p))
        nvect <- as.vector(rep(1,n))
        nk_one <- matrix(1,n,p)
        nk_zero <- matrix(0,n,p)
        kk_zero <- matrix(0,p,p)
        kk_one <- diag(p)
	epsv<-eps1*kvect
	epsn<-eps*nvect
        E_SX_n <- matrix(0,p,l)
        E_SSXX_n <- list()

        L <- (0.5*XX^(.5))*nk_one
        L <- L/sqrt(sum(L*L))

	for (i in 1:cyc){
                ipsi <- 1/Psi
		LPsi<-diag(ipsi)%*%L
		LPsiL<-crossprod(L,LPsi)
 		sum1<- nk_zero
		sum2<- eps2*kk_one
 		for (j in 1:l){
			tmp <- solve(LPsiL+diag(lapla[j,]))
			x_j <- as.vector(X[,j])
			e_sx_n <- as.vector(tcrossprod(tmp,LPsi)%*%x_j)
			#e_sx_n[which(e_sx_n<0)] <- 0
			e_ssxx_n <-  tmp + tcrossprod(e_sx_n,e_sx_n)
			sum1 <- sum1 +  tcrossprod(x_j,e_sx_n)
			sum2 <- sum2 + e_ssxx_n
			lapla[j,] <- (epsv+diag(e_ssxx_n))^(-spz)
                        lapla[j,which( lapla[j,]<1.0)] <- 1.0
		}
                L <- sum1%*%solve(sum2)

                Psi <- epsn+abs(XX - diag(tcrossprod(L,sum1)/n))

                for (j in 1:p)
                {
                    L[,j] <- projfunc(s=as.vector(L[,j]),k1=L1a,k2=1.0)
                }

                if (i %% 20==0) { print(i)}
	}

        ipsi <- 1/Psi
        LPsi<-diag(ipsi)%*%L
        LPsiL<-crossprod(L,LPsi)
        for (j in 1:l){
            tmp <- solve(LPsiL+diag(lapla[j,]))
            x_j <- as.vector(X[,j])
            e_sx_n <- as.vector(tcrossprod(tmp,LPsi)%*%x_j)
            E_SX_n[,j] <- e_sx_n
            E_SSXX_n[[j]] <-  tmp + tcrossprod(e_sx_n,e_sx_n)
        }


        # INI call for biclusters

        vz <- iin*apply(E_SX_n,1,function(x) sum(x^2))


        vz <- sqrt(vz+1e-10)

        ivz <- 1/vz

        if(length(ivz)==1) {
            nZ <- ivz*E_SX_n

            nL <- vz*L
        }
        else {
            nZ <- ivz*E_SX_n

            nL <- t(vz*t(L))
        }







        ini <- matrix(0,l,(p+1))
        avini <- as.vector(rep(0.0,(p+1)))
        xavini <- as.vector(rep(0.0,(l+1)))

        idp <- diag(p)
        ipsi <- 1/Psi

        for (j in 1:l){
                ppL <- crossprod(nL,ipsi*nL)
                mat <- idp + ppL/lapla[j,]
                ini[j,p+1] <- log(det(mat))

                for (i in 1:p){
                    ini[j,i] <- log(1+(1/lapla[j,i])*crossprod(nL[,i],ipsi*nL[,i]))
                }
            }
        for (i in 1:p){
            avini[i] <- sum(ini[,i])
        }
        for (i in 1:l){
            xavini[i] <- ini[i,p+1]
        }

        ss <- sum(ini[,p+1])
        xavini[l+1] <- ss
        avini[p+1] <- ss



        Lz=nL%*%nZ

        rownames(nL) <- rowna
        colnames(nL) <- colnames(nL, do.NULL = FALSE, prefix = "bicluster")
        clnames <- colnames(nL)

        rownames(nZ) <- clnames
        colnames(nZ) <- colna

        M <- diag(p)
        rownames(M) <- clnames
        colnames(M) <- clnames

        rownames(Lz) <- rowna
        colnames(Lz) <- colna

        U <- X-Lz

        rownames(U) <- rowna
        colnames(U) <- colna


return(new('Factorization', parameters=list("fabiasp",cyc,alpha,NULL,spz,p,NULL,NULL,NULL,NULL,norm,center,NULL),n=n,p1=p,p2=p,l=l,center=cent,scaleData=scaleData,X=X,L=nL,Z=nZ,M=M,LZ=Lz,U=U,avini=avini,xavini=xavini,ini=ini,Psi=Psi,lapla=lapla))

}



fabiaVersion <- function() {
  version <- packageDescription("fabia",fields="Version")
  cat('\nFABIA Package Version ', version, '\n')
  cat('\nCopyright (c) 2009 by Sepp Hochreiter\n\n')
}


estimateMode <- function(X,maxiter=50,tol=0.001,alpha=0.1,a1=4.0,G1=FALSE) {

        if (missing(X)) {
            stop("Data matrix X missing for mode. Stopped.")
        }

        if (!is.matrix(X)) {
            X <- as.matrix(X)
        }

        if (!is.numeric(X)) {
            stop("Data matrix X must be numeric for mode. Stopped.")
        }




    l=ncol(X)
    n=nrow(X)


    iin <-  1.0/l
    u <- apply(X, 1, median)
    xu <- X - u
    XX <-  sqrt(iin*apply(xu,1,function(x) sum(x^2))+0.001*as.vector(rep(1,n)))
    dXX <- 1/XX
    X <- dXX*X

    u <- apply(X, 1, median)
    gxu <- 10.0*tol
    iter <- 1
    alpha <- alpha/l

    xu <- X - u

    if (G1) {
        while (abs(gxu) > tol && iter < maxiter) {
            gxu <- apply(xu, 1, function(x) alpha*sum(tanh(a1 * x)))
            u <- u + gxu
            xu <- X - u
            iter <- iter + 1
        }
    } else {
        while (abs(gxu) > tol && iter < maxiter) {
            gxu <- apply(xu, 1, function(x) alpha*sum(x * exp(-a1*(x^2)/2)))
            u <- u + gxu
            xu <- X - u
            iter <- iter + 1
        }
    }

    u <- XX*u
    xu <- XX*xu

    return(list(u=u,xu=xu))
}

myImagePlot <- function(x,xLabels=NULL, yLabels=NULL, zlim=NULL, title=NULL){

        if (missing(x)) {
            stop("Data matrix x missing. Stopped.")
        }

        if (!is.matrix(x)) {
            x <- as.matrix(x)
        }

        if (!is.numeric(x)) {
            stop("Data matrix x must be numeric. Stopped.")
        }



     min <- min(x)
     max <- max(x)

      if( !is.null(zlim) ){
       min <- zlim[1]
       max <- zlim[2]
    }

    if( !is.null(yLabels) ){
       yLabels <- c(yLabels)
    } else {
       yLabels <- rownames(x)
   }

    if( !is.null(xLabels) ){
      xLabels <- c(xLabels)
    } else {
     xLabels <- colnames(x)
    }

     if( is.null(title) ){
     title <- c()
    }

# check for null values
if( is.null(xLabels) ){
   xLabels <- c(1:ncol(x))
}
if( is.null(yLabels) ){
   yLabels <- c(1:nrow(x))
}

#layout(matrix(data=c(1,2), nrow=1, ncol=2), widths=c(4,1), heights=c(1,1))
layout(matrix(data=c(1,2), nrow=1, ncol=2), widths=c(7,1), heights=c(1,1))

 # Red and green range from 0 to 1 while Blue ranges from 1 to 0
 ColorRamp <- rgb( seq(0,1,length=256),  # Red
                   seq(0,1,length=256),  # Green
                   seq(1,0,length=256))  # Blue
 ColorLevels <- seq(min, max, length=length(ColorRamp))

 # Reverse Y axis
 reverse <- nrow(x) : 1
 yLabels <- yLabels[reverse]
 x <- x[reverse,]

 # Data Map
# par(mar = c(3,5,2.5,2))
 par(mar = c(3,5,2.5,1))
 image(1:length(xLabels), 1:length(yLabels), t(x), col=ColorRamp, xlab="",
 ylab="", axes=FALSE, zlim=c(min,max))
 if( !is.null(title) ){
    title(main=title)
 }
axis(BELOW<-1, at=1:length(xLabels), labels=xLabels, cex.axis=0.7)
 axis(LEFT <-2, at=1:length(yLabels), labels=yLabels, las= HORIZONTAL<-1,
 cex.axis=0.7)

 # Color Scale
# par(mar = c(3,2.5,2.5,2))
 par(mar = c(3,2,2.5,1))
 image(1, ColorLevels,
      matrix(data=ColorLevels, ncol=length(ColorLevels),nrow=1),
      col=ColorRamp,
      xlab="",ylab="",
      xaxt="n")

 layout(1)
}



plotBicluster <- function(r,p,opp=FALSE,zlim=NULL,title=NULL,which=c(1, 2)){



        if (missing(r)) {
            stop("r, the result of extract_bic, is missing. Stopped.")
        }
        if (missing(p)) {
            stop("The bicluster to plot is missing. Stopped.")
        }



        x <- r$X
        if (!is.matrix(x)) {
            x <- as.matrix(x)
        }

        if (!is.numeric(x)) {
            stop("Data matrix X must be numeric. Stopped.")
        }
        if (!is.numeric(r$np)) {
            stop("Number of biclusters must be numeric. Stopped.")
        }


        if (p>r$np) {
            stop("Bicluster number is too large. Stopped.")
        }



    if (length(which) > 1 && dev.interactive()) {
        devAskNewPage(ask = TRUE)
    }

   showf <- c(FALSE, FALSE)
    showf[which] <- TRUE




     n <- nrow(x)

     np <- r$np

     l <- ncol(x)

     if (!opp) {
         if (r$bic[[p]][1]<1) {
            stop("No genes in bicluster. Stopped.")
         }
         if (r$bic[[p]][2]<1) {
            stop("No samples in bicluster. Stopped.")
         }

        observations <- unlist(r$bic[p,3])
         samples <- unlist(r$bic[p,5])
     } else {
         if (r$bicopp[[p]][1]<1) {
            stop("No genes in bicluster. Stopped.")
         }
         if (r$bicopp[[p]][2]<1) {
            stop("No samples in bicluster. Stopped.")
         }
         observations <- unlist(r$bicopp[p,3])
         samples <- unlist(r$bicopp[p,5])
     }

     if (length(observations)<1) {
         stop("No genes in bicluster. Stopped.")
     }

     if (length(samples)<1) {
         stop("No samples in bicluster. Stopped.")
     }

     min <- min(x)
     max <- max(x)

   if( !is.null(zlim) ){
       min <- zlim[1]
       max <- zlim[2]
    }

    yLabels <- rownames(x)
    xLabels <- colnames(x)


     if( is.null(title) ){
     title <- c()
    }



 # Reverse Y axis
 reverse <- n : 1
 yLabels <- yLabels[reverse]
 x <- x[reverse,]

     obs <- match(observations,yLabels)
     samp <- match(samples,xLabels)

    ybl <- length(obs)
    if (ybl == 0) {
      obs <- match(observations,as.character(n:1))
      ybl <- length(obs)

    }
    if (ybl == 0) {
      obs <- match(observations,c(n:1))
      ybl <- length(obs)

    }

    xbl <- length(samp)
    if (xbl == 0) {
      samp <- match(samples,as.character(1:l))
      xbl <- length(samp)

    }
    if (xbl == 0) {
      samp <- match(samples,c(1:l))
      xbl <- length(samp)

    }



    pmL <- diag(n)
    for (i in 1:ybl){
        ii <- n-i+1
        pmL[ii,ii] <- 0
        pmL[ii,obs[i]] <- 1
        pmL[obs[i],obs[i]] <- 0
        pmL[obs[i],ii] <- 1
        tmp <- yLabels[ii]
        yLabels[ii] <- yLabels[obs[i]]
        yLabels[obs[i]] <- tmp
    }

    pmZ <- diag(l)
    for (i in 1:xbl){
        pmZ[i,i] <- 0
        pmZ[i,samp[i]] <- 1
        pmZ[samp[i],samp[i]] <- 0
        pmZ[samp[i],i] <- 1
        tmp <- xLabels[i]
        xLabels[i] <- xLabels[samp[i]]
        xLabels[samp[i]] <- tmp
   }

    x <- pmL%*%x%*%pmZ

    rownames(x) <-  yLabels
    colnames(x) <-  xLabels


    if (n>100)
    {
        lll <- 12

        yl <- c(1,round(n*(1:lll)/lll))

        yll <- rep("",n)

        yll[yl] <- rownames(x)[yl]

        yLabels <- yll
    }
    for (i in 1:ybl){
        ii <- n-i+1
        yLabels[ii] <- rownames(x)[ii]
    }

if (showf[1]){

 layout(matrix(data=c(2,1), nrow=1, ncol=2), widths=c(7,1), heights=c(1,1))

 # Red and green range from 0 to 1 while Blue ranges from 1 to 0
 ColorRamp <- rgb( seq(0,1,length=256),  # Red
                   seq(0,1,length=256),  # Green
                   seq(1,0,length=256))  # Blue
 ColorLevels <- seq(min, max, length=length(ColorRamp))

 # Color Scale
 par(mar = c(3,2,2.5,1))
 image(1, ColorLevels,
      matrix(data=ColorLevels, ncol=length(ColorLevels),nrow=1),
      col=ColorRamp,
      xlab="",ylab="",
      xaxt="n")




 # Data Map
 par(mar = c(3,5,2.5,1))
 image(1:length(xLabels), 1:length(yLabels), t(x), col=ColorRamp, xlab="",
 ylab="", axes=FALSE, zlim=c(min,max))
 if( !is.null(title) ){
    title(main=title)
 }
axis(BELOW<-1, at=1:length(xLabels), labels=xLabels, cex.axis=0.7)
 axis(LEFT <-2, at=1:length(yLabels), labels=yLabels, las= HORIZONTAL<-1,
 cex.axis=0.7)

#rect(xleft, ybottom, xright, ytop, density = NULL, angle = 45,
#     col = NA, border = NULL, lty = par("lty"), lwd = par("lwd"),
#     ...)

rect(xleft=0.6, ybottom= n-ybl, xright=xbl+1, ytop=n+0.3,lwd=3,border="red")

layout(1)

}


if (showf[2]){

layout(matrix(data=c(2,1), nrow=1, ncol=2), widths=c(7,1), heights=c(1,1))

 ColorRamp <- rgb( seq(0,1,length=256),  # Red
                   seq(0,1,length=256),  # Green
                   seq(1,0,length=256))  # Blue
 ColorLevels <- seq(min, max, length=length(ColorRamp))

# Color Scale
 par(mar = c(3,2,2.5,1))
 image(1, ColorLevels,
      matrix(data=ColorLevels, ncol=length(ColorLevels),nrow=1),
      col=ColorRamp,
      xlab="",ylab="",
      xaxt="n")


 # Data Map
 par(mar = c(3,5,2.5,1))
 image(1:xbl, 1:ybl, t(x[(n-ybl+1):n,1:xbl]), col=ColorRamp, xlab="",
 ylab="", axes=FALSE, zlim=c(min,max))
 if( !is.null(title) ){
    title(main=title)
 }
axis(BELOW<-1, at=1:xbl, labels=xLabels[1:xbl], cex.axis=0.7)
 axis(LEFT <-2, at=1:ybl, labels=yLabels[(n-ybl+1):n], las= HORIZONTAL<-1,
 cex.axis=0.7)

 layout(1)

}
devAskNewPage(ask = FALSE)
}




#########################
##Data
make_fabi_data <- function(n,l,p,f1,f2,of1,of2,sd_noise,sd_z_noise,mean_z,sd_z,sd_l_noise,mean_l,sd_l){

    if(!is.numeric(n)) {
        stop("n must be numeric")
    }
    if(!is.numeric(l)) {
        stop("l must be numeric")
    }

    if(!is.numeric(p)) {
        stop("p must be numeric")
    }
    if(!is.numeric(f1)) {
        stop("f1 must be numeric")
    }



    za <- floor(runif(p)*(l/f1)+of1)
    la <- floor(runif(p)*(n/f2)+of2)
    ZC <- list()
    LC <- list()

    X <- matrix(rnorm(l*n,mean = 0, sd = sd_noise),n,l)
    Y <- matrix(0,n,l)
    rownames(X) <- rownames(X, do.NULL = FALSE, prefix = "gene")
    colnames(X) <- colnames(X, do.NULL = FALSE, prefix = "sample")
    rownames(Y) <- rownames(X)
    colnames(Y) <- colnames(X)

for (i in 1:p){

        zj <- rnorm(l,mean = 0, sd = sd_z_noise)
        z0 <- rep(0,l)
        zf <- floor(runif(za[i])*l)
        ZC[[i]] <- zf
        z1 <- rnorm(za[i],mean = mean_z, sd = sd_z)
        zj[zf] <- z1
        z0[zf] <- z1

        lj <- rnorm(n,mean = 0, sd = sd_l_noise)
        l0 <- rep(0,n)
        lf <- floor(runif(la[i])*n)
        LC[[i]] <- lf
        l1 <- rnorm(la[i],mean = mean_l, sd = sd_l)
        lsign <- 2*round(runif(la[i]))-1
        l1 <- l1*lsign
        lj[lf] <- l1
        l0[lf] <- l1

        X <- X + lj%*%t(zj)
        Y <- Y + l0%*%t(z0)



    }


return(list(X=X,Y=Y,ZC=ZC,LC=LC))
}

#########################
##Blocks
make_fabi_data_blocks <- function(n,l,p,f1,f2,of1,of2,sd_noise,sd_z_noise,mean_z,sd_z,sd_l_noise,mean_l,sd_l){

    if(!is.numeric(n)) {
        stop("n must be numeric")
    }
    if(!is.numeric(l)) {
        stop("l must be numeric")
    }

    if(!is.numeric(p)) {
         stop("p must be numeric")
    }
    if(!is.numeric(f1)) {
        stop("f1 must be numeric")
    }


    za <- floor(runif(p)*(l/f1)+of1)
    la <- floor(runif(p)*(n/f2)+of2)
    ZC <- list()
    LC <- list()


    X <- matrix(rnorm(l*n,mean = 0, sd = sd_noise),n,l)
    Y <- matrix(0,n,l)

    rownames(X) <- rownames(X, do.NULL = FALSE, prefix = "gene")
    colnames(X) <- colnames(X, do.NULL = FALSE, prefix = "sample")
    rownames(Y) <- rownames(X)
    colnames(Y) <- colnames(X)

   for (i in 1:p){


        zj <- rnorm(l,mean = 0, sd = sd_z_noise)
        z0 <- as.vector(rep(0,l))

        tf <- floor(runif(1)*(l-za[i]))
        zf <- tf:(tf+za[i]-1)
        ZC[[i]] <- zf
        z1 <- rnorm(za[i],mean = mean_z, sd = sd_z)
        zj[zf] <- z1
        z0[zf] <- z1

        lj <- rnorm(n,mean = 0, sd = sd_l_noise)
        l0 <- as.vector(rep(0,n))
        uf <- floor(runif(1)*(n-la[i]))
        lf <- uf:(uf+la[i]-1)
        LC[[i]] <- lf
        l1 <- rnorm(la[i],mean = mean_l, sd = sd_l)
        lsign <- 2*round(runif(la[i]))-1
        l1 <- l1*lsign
        lj[lf] <- l1
        l0[lf] <- l1

        X <- X + lj%*%t(zj)
        Y <- Y + l0%*%t(z0)


    }
    return(list(X=X,Y=Y,ZC=ZC,LC=LC))
}

#########################
##Data
make_fabi_data_pos <- function(n,l,p,f1,f2,of1,of2,sd_noise,sd_z_noise,mean_z,sd_z,sd_l_noise,mean_l,sd_l){

    if(!is.numeric(n)) {
        stop("n must be numeric")
    }
    if(!is.numeric(l)) {
        stop("l must be numeric")
    }

    if(!is.numeric(p)) {
        stop("p must be numeric")
    }
    if(!is.numeric(f1)) {
        stop("f1 must be numeric")
    }



    za <- floor(runif(p)*(l/f1)+of1)
    la <- floor(runif(p)*(n/f2)+of2)
    ZC <- list()
    LC <- list()

    X <- matrix(rnorm(l*n,mean = 0, sd = sd_noise),n,l)
    Y <- matrix(0,n,l)

    rownames(X) <- rownames(X, do.NULL = FALSE, prefix = "gene")
    colnames(X) <- colnames(X, do.NULL = FALSE, prefix = "sample")
    rownames(Y) <- rownames(X)
    colnames(Y) <- colnames(X)

   for (i in 1:p){

        zj <- rnorm(l,mean = 0, sd = sd_z_noise)
        z0 <- rep(0,l)
        zf <- floor(runif(za[i])*l)
        ZC[[i]] <- zf
        z1 <- rnorm(za[i],mean = mean_z, sd = sd_z)
        zj[zf] <- z1
        z0[zf] <- z1

        lj <- rnorm(n,mean = 0, sd = sd_l_noise)
        l0 <- rep(0,n)
        lf <- floor(runif(la[i])*n)
        LC[[i]] <- lf
        l1 <- rnorm(la[i],mean = mean_l, sd = sd_l)
        lj[lf] <- l1
        l0[lf] <- l1

        X <- X + lj%*%t(zj)
        Y <- Y + l0%*%t(z0)


    }
    return(list(X=X,Y=Y,ZC=ZC,LC=LC))
}

#########################
##Blocks
make_fabi_data_blocks_pos<- function(n,l,p,f1,f2,of1,of2,sd_noise,sd_z_noise,mean_z,sd_z,sd_l_noise,mean_l,sd_l){

    if(!is.numeric(n)) {
        stop("n must be numeric")
    }
    if(!is.numeric(l)) {
        stop("l must be numeric")
    }

    if(!is.numeric(p)) {
         stop("p must be numeric")
    }
    if(!is.numeric(f1)) {
        stop("f1 must be numeric")
    }


    za <- floor(runif(p)*(l/f1)+of1)
    la <- floor(runif(p)*(n/f2)+of2)
    ZC <- list()
    LC <- list()


    X <- matrix(rnorm(l*n,mean = 0, sd = sd_noise),n,l)
    Y <- matrix(0,n,l)


    rownames(X) <- rownames(X, do.NULL = FALSE, prefix = "gene")
    colnames(X) <- colnames(X, do.NULL = FALSE, prefix = "sample")
    rownames(Y) <- rownames(X)
    colnames(Y) <- colnames(X)

    for (i in 1:p){


        zj <- rnorm(l,mean = 0, sd = sd_z_noise)
        z0 <- as.vector(rep(0,l))

        tf <- floor(runif(1)*(l-za[i]))
        zf <- tf:(tf+za[i]-1)
        ZC[[i]] <- zf
        z1 <- rnorm(za[i],mean = mean_z, sd = sd_z)
        zj[zf] <- z1
        z0[zf] <- z1

        lj <- rnorm(n,mean = 0, sd = sd_l_noise)
        l0 <- as.vector(rep(0,n))
        uf <- floor(runif(1)*(n-la[i]))
        lf <- uf:(uf+la[i]-1)
        LC[[i]] <- lf
        l1 <- rnorm(la[i],mean = mean_l, sd = sd_l)
        lj[lf] <- l1
        l0[lf] <- l1

        X <- X + lj%*%t(zj)
        Y <- Y + l0%*%t(z0)


    }
    return(list(X=X,Y=Y,ZC=ZC,LC=LC))
}


# mfsc - matrix factorization with sparseness constraints
#
# Written by Sepp Hochreiter according to
# Patrik O. Hoyer
# 'Non-negative matrix factorization with sparseness constraints'
# Journal of Machine Learning Research  5:1457-1469, 2004.
#
#
#
# SYNTAX:
# res <- mfsc(X,p=5,cyc=100,sL=0.6,sZ=0.6,center=2,norm=1)
# res$L
# res$Z
#
# INPUTS:
# X          - data matrix
# p       - number of components (inner dimension of factorization)
# sL         - sparseness of L, in [0,1]
# sZ         - sparseness of Z, in [0,1]
# cyc    - maximal number of iterations
#
# Note: Sparseness is measured on the scale [0,1] where 0 means
# completely distributed and 1 means ultimate sparseness.
#
# NOTE: There is NO CONVERGENCE CRITERION.
#


mfsc <- function(X,p=5,cyc=100,sL=0.6,sZ=0.6,center=2,norm=1) {

        if (missing(X)) {
            stop("Data matrix X missing. Stopped.")
        }

        if (!is.matrix(X)) {
            X <- as.matrix(X)
        }

        if (!is.numeric(X)) {
            stop("Data matrix X must be numeric. Stopped.")
        }

        l=ncol(X)
        n=nrow(X)

        rownames(X) <- rownames(X, do.NULL = FALSE, prefix = "gene")
        colnames(X) <- colnames(X, do.NULL = FALSE, prefix = "sample")
        rowna <- rownames(X)
        colna <- colnames(X)


       cat("Running MFSC on ",n,"x",l," matrix with\n")
       cat("   Number of biclusters ---------------- p: ",p,"\n")
       cat("   Number of maximal iterations ------ cyc: ",cyc,"\n")
       cat("   Loading sparseness projection ------ sL: ",sL,"\n")
       cat("   Factor sparseness projection ------- sZ: ",sZ,"\n")
       if (center>0) {
            if (center<2) {
       cat("   Centering ---------------------- center:  1  = mean\n")
            } else {
                if (center<3) {
       cat("   Centering ---------------------- center:  2  = median\n")
                } else {
       cat("   Centering ---------------------- center: ", center,"  = mode\n")
                }
            }
        } else {
       cat("   Centering ---------------------- center: ", center," = no centering\n")
        }
       if (norm>0) {
           if (norm > 1) {
       cat("   Scaling to variance one: --------- norm: ",norm," = Yes\n")
           } else {
       cat("   Quantile scaling (0.75-0.25): ---- norm: ",norm," = Yes\n")
           }} else {
       cat("   Scaling -------------------------- norm: ",norm," = No\n")
       }

        iin <-  1.0/l

        ini <- matrix(1,l,(p+1))
        avini <- as.vector(rep(1.0,(p+1)))
        xavini <- as.vector(rep(1.0,(l+1)))

        cent <- as.vector(rep(0.0,n))
        if (center>0) {
            if (center<2) {
                cent <- apply(X, 1, mean)
            } else {
                if (center<3) {
                    cent <- apply(X, 1, median)
                } else {
                    cent <- estimateMode(X)
                }
            }
        }

        X <- X - cent

        XX <- as.vector(rep(1,n))

        if (norm>0) {
            if (norm>1)
            {
                scaleData <-  1/sqrt(iin*apply(X,1,function(x) sum(x^2))+0.001*XX)
            }
            else
            {
                scaleData <-  1/sqrt(apply(X,1,function(x) {quantile(x,0.75) - quantile(x,0.25)})+0.001*XX)
            }
            X <- scaleData*X
        } else {
            scaleData <- as.vector(rep(1.0,n))
        }


# Create initial matrices
L <- matrix (rnorm(n*p),nrow=n,ncol=p)
Z <- matrix (rnorm(l*p),nrow=p,ncol=l)



# Make initial matrices have correct sparseness
L1a <- sqrt(n)-(sqrt(n)-1)*sL
L1s <- sqrt(l)-(sqrt(l)-1)*sZ

for (i in 1:p)
  {
    L[,i] <- projfunc(s=as.vector(L[,i]),k1=L1a,k2=1.0)
    Z[i,] <- projfunc(s=as.vector(Z[i,]),k1=L1s,k2=1.0)
  }


# Calculate initial objective
objhistory <- sum((X-L%*%Z)^2)



# Initial stepsizes
stepsizeL <- 1.0
stepsizeZ <- 1.0



# Start iteration
for (i in 1:cyc)
  {


    # Save old values
    Lold <- L
    Zold <- Z

    # ----- Update Z ---------------------------------------


	# Gradient for Z
	dZ <- t(L)%*%(L%*%Z-X)

        begobj <- objhistory

        newobj <- 2.0*begobj




     # Make sure we decrease the objective!
	while (newobj>begobj) {


         # Take step in direction of negative gradient, and project
          Znew <- Z - stepsizeZ*dZ
	# Make initial matrices have correct sparseness
          for (j in 1:p)
            {
              Znew[j,] <- projfunc(s=as.vector(Znew[j,]),k1=L1s,k2=1.0)
            }


         # Calculate new objective
	    newobj <- sum((X-L%*%Znew)^2)

	    # If the objective decreased, we can continue...
            stepsizeZ <- stepsizeZ/2.0
            if (stepsizeZ<1e-10) {

                rownames(L) <- rowna
                colnames(L) <- colnames(L, do.NULL = FALSE, prefix = "bicluster")
                clnames <- colnames(L)

                rownames(Z) <- clnames
                colnames(Z) <- colna

                M <- diag(p)
                rownames(M) <- clnames
                colnames(M) <- clnames
                LZ <- L%*%Z

                rownames(LZ) <- rowna
                colnames(LZ) <- colna

                U <- X-LZ

                rownames(U) <- rowna
                colnames(U) <- colna

            return(new('Factorization', parameters=list("mfsc",cyc,NULL,NULL,NULL,p,sL,sZ,NULL,NULL,norm,center,NULL),n=n,p1=p,p2=p,l=l,center=cent,scaleData=scaleData,X=X,L=L,Z=Z,M=M,LZ=LZ,U=U,avini=avini,xavini=xavini,ini=ini,Psi=as.vector(1),lapla=as.matrix(1)))
            }
        }

	# Slightly increase the stepsize
	stepsizeZ <- stepsizeZ*1.2
	Z <- Znew



    # ----- Update L ---------------------------------------


	# Gradient for L
	dL <- (L%*%Z-X)%*%t(Z)
	begobj <- sum((X-L%*%Z)^2)

        newobj <- 2*begobj

	# Make sure we decrease the objective!
	while (newobj>begobj) {



	    # Take step in direction of negative gradient, and project
	    Lnew <- L - stepsizeL*dL
	    norms <- sqrt(sum(Lnew^2))
            for (j in 1:p)
              {
                Lnew[,j] <- projfunc(s=as.vector(Lnew[,j]),k1=L1a,k2=1.0)
              }

	    # Calculate new objective
	    newobj <- sum((X-Lnew%*%Z)^2)

	    stepsizeL <- stepsizeL/2

            if (stepsizeZ<1e-10) {

                rownames(L) <- rowna
                colnames(L) <- colnames(L, do.NULL = FALSE, prefix = "bicluster")
                clnames <- colnames(L)

                rownames(Z) <- clnames
                colnames(Z) <- colna

                M <- diag(p)
                rownames(M) <- clnames
                colnames(M) <- clnames
                LZ <- L%*%Z

                rownames(LZ) <- rowna
                colnames(LZ) <- colna

                U <- X-LZ

                rownames(U) <- rowna
                colnames(U) <- colna

                return(new('Factorization', parameters=list("mfsc",cyc,NULL,NULL,NULL,p,sL,sZ,NULL,NULL,norm,center,NULL),n=n,p1=p,p2=p,l=l,center=cent,scaleData=scaleData,X=X,L=L,Z=Z,M=M,LZ=LZ,U=U,avini=avini,xavini=xavini,ini=ini,Psi=as.vector(1),lapla=as.matrix(1)))
            }


          }

	# Slightly increase the stepsize
	stepsizeL <- stepsizeL*1.2
	L <- Lnew


    # Calculate objective
    newobj <- 0.5*sum(sum((X-L%*%Z)^2))
    objhistory <- newobj



  }
}




# Solves the following problem:
# Given a vector s, find the vector v having sum(abs(v))=k1
# and sum(v^2)=k2 which is closest to s in the euclidian sense.


projfunc <- function(s, k1, k2) {


  N <- length(s)

  isneg <- as.vector(rep(0,N))
  isneg[which(s<0)] <- 1
  ones <- as.vector(rep(1,N))

  s <- abs(s)


# Start by projecting the point to the sum constraint hyperplane
  v <- s + (k1-sum(s))/N

# Initialize zerocoeff (initially, no elements are assumed zero)
  # zerocoeff <- which(v<=0)
  # v[zerocoeff] <- 0

  zerocoeff <- vector(mode = "integer", length = 0)


  j <- 0
  ende <- 0
  while (ende<1) {

    # This does the proposed projection operator
    midpoint <- ones*k1/(N-length(zerocoeff))
    midpoint[zerocoeff] <- 0
    w <- v-midpoint
    a <- sum(w^2)
    b <- 2*crossprod(w,v)
    c <- sum(v^2)-k2
    t <- b^2-4*a*c
    if (t<0) {t <- 0}
    if (a<1e-10) {a <- 1e-10}

    alphap <- (-b+sqrt(t))/(2*a)
    v <- alphap*w + v

    if ((length(which(v<0))==0)&&(j>1)) {
      ende <- 2
    }
    else {

      j <- j+1

    # Set negs to zero, subtract appropriate amount from rest
      zerocoeff <- which(v<=0)
      v[zerocoeff] <- 0
      tempsum <- sum(v)
      tta <- N - length(zerocoeff)
      if (tta>0)  {
       v <- v + (k1-tempsum)/tta
       }
      v[zerocoeff] <- 0
     }
  }

  v <- (-2*isneg + ones)*v

  return(v)

}


##########################################################
extract_plot <- function(fact,thresZ=0.5,ti="",thresL=NULL,Y=NULL,which=c(1,2,3,4,5,6,7,8)){

        if (missing(fact)) {
            stop("Object fact of class Factorization is missing. Stopped.")
        }

        if (is(fact) != "Factorization") {
            stop("Object fact is not of the class Factorization. Stopped.")
        }



    if (length(which) > 1 && dev.interactive()) {
      devAskNewPage(ask = TRUE)
    }

    if ((length(which) == 2) && (!is.null(Y)) && (which[1]==1)) {

      devAskNewPage(ask = FALSE)
    }


    showf <- c(FALSE, FALSE,FALSE, FALSE,FALSE, FALSE,FALSE, FALSE)
    showf[which] <- TRUE





    X <- as.matrix(fact@X)
    nL <- as.matrix(fact@L)
    nZ <- as.matrix(fact@Z)

    l=ncol(X)
    n=nrow(X)


    p <- ncol(nL)




    lll <- 12

    yl <- c(1,round(n*(1:lll)/lll))

    yll <- rep("",n)

    yll[yl] <- rownames(X)[yl]

    LZ <- nL%*%nZ

    tt <- paste("(",as.character(n)," genes, ",as.character(l)," samples, ",as.character(p)," biclusters )")

    if (!is.null(Y) && showf[1]){
      myImagePlot(Y,title=paste(ti,": noise free data\n",tt,sep=""),yLabels= yll)
    }

    if (showf[2]){
        myImagePlot(X,title=paste(ti,": data\n",tt,sep=""),yLabels= yll)
    }

    if (showf[3]){
        myImagePlot(LZ,title=paste(ti,": reconstructed data\n",tt,sep=""),yLabels= yll)
    }
    if (showf[4]){
        myImagePlot(LZ-X,title=paste(ti,": error\n",tt,sep=""),yLabels= yll)
    }



    if (showf[5]){
        myImagePlot(abs(nL),title=paste(ti,": absolute loadings\n",tt,sep=""),yLabels= yll)
    }
    if (showf[6]){
        myImagePlot(abs(nZ),title=paste(ti,": absolute factors\n",tt,sep=""))
    }




    hnzc <- kmeans(t(nZ),p)

    hnzs <- sort(hnzc$cluster,index.return = TRUE)

    xLabels <- colnames(X)
    pmZ <- matrix(0,l,l)
    for (i in 1:l){
         pmZ[hnzs$ix[i],i] <- 1
         tmp <- xLabels[i]
         xLabels[i] <- xLabels[hnzs$ix[i]]
         xLabels[hnzs$ix[i]] <- tmp
    }




    hnLc <- kmeans(nL,p)

    hnLs <- sort(hnLc$cluster,index.return = TRUE)

    biclustx <- list()
    biclusty <- list()
    for (i in 1:p){

        biclustx[[i]] <- hnLs$ix[which(hnLs$x==i)]
        biclusty[[i]] <- hnzs$ix[which(hnzs$x==i)]

    }

    biclust <- cbind(biclustx,biclusty)

    pmL <- matrix(0,n,n)
    yLabels <- rownames(X)
    for (i in 1:n){
         pmL[i,hnLs$ix[i]] <- 1
         tmp <- yLabels[i]
         yLabels[i] <- yLabels[hnLs$ix[i]]
         yLabels[hnLs$ix[i]] <- tmp
     }


    lll <- 12

    yl <- c(1,round(n*(1:lll)/lll))

    yll <- rep("",n)

    yll[yl] <- yLabels[yl]

    reconstr <- pmL%*%nL%*%nZ%*%pmZ

    rownames(reconstr) <- yLabels
    colnames(reconstr) <- xLabels

    if (showf[7]){
        myImagePlot(reconstr,title=paste(ti,": reconstructed matrix sorted\n",tt,sep=""), yLabels= yll)
    }

    reord <- pmL%*%as.matrix(X)%*%pmZ

    rownames(reord) <- yLabels
    colnames(reord) <- xLabels

     if (showf[8]){
         myImagePlot(reord,title=paste(ti,": original matrix sorted\n",tt,sep=""), yLabels= yll)
     }

    devAskNewPage(ask = FALSE)

    return(list(biclust=biclust,pmZ=pmZ,pmL=pmL,L=nL,Z=nZ,Xord=reord))

}


##########################################################
extract_bic <- function(fact,thresZ=0.5,thresL=NULL)
{

    if (missing(fact)) {
        stop("Object fact of class Factorization is missing. Stopped.")
    }

    if (is(fact) != "Factorization") {
        stop("Object fact is not of the class Factorization. Stopped.")
    }



    nL <- as.matrix(fact@L)
    nZ <- as.matrix(fact@Z)
    X <- as.matrix(fact@X)

    n <- nrow(nL)

    p <- ncol(nL)

    l <- ncol(nZ)



    binn <- list()
    binp <- list()
    bixv <- list()
    bixn <- list()
    numng <- list()

    numnp <- list()
    biyp <- list()
    biypv <- list()
    biypn <- list()

    numnn <- list()
    biyn <- list()
    biynv <- list()
    biynn <- list()
    if (is.null(rownames(X)))
    {
        rownames(X) <- as.character(1:n)
    }
    gene_names <- rownames(X)
    rownames(nL) <- gene_names

    if (is.null(colnames(X)))
    {
       colnames(X) <- as.character(1:l)
    }
    sample_names <- colnames(X)
    colnames(nZ) <- sample_names

    if (is.null(thresL)) {
        mom <- 0
        for (i in 1:p) {
            tmom <- as.numeric(tcrossprod(nL[,i],nZ[i,]))
            mom <- mom + sum(tmom^2)
        }
        mom <- mom/(length(tmom)*p)

        thresL <- sqrt(mom)/thresZ
    }


    for (i in 1:p){

        snl <- sort(abs(nL[,i]),decreasing = TRUE,index.return = TRUE)
        gene_namest <- gene_names[snl$ix]
        sL <- nL[snl$ix,i]


        numng[[i]] <- as.vector(which(abs(nL[,i])>thresL))
        cho <- which(abs(sL)>thresL)
        bixv[[i]] <- sL[cho]
        bixn[[i]] <- gene_namest[cho]


        snz <- sort(abs(nZ[i,]),decreasing = TRUE,index.return = TRUE)
        sample_namest <- sample_names[snz$ix]
        sz <- nZ[i,snz$ix]

        chop <- which(sz>thresZ)
        ss2 <- sum(abs(sz[chop]))
        chon <- which(sz< -thresZ)
        ss3 <- sum(abs(sz[chon]))

        if (ss2>=ss3) {
            binp[[i]] <- c(length(cho),length(chop))
            numnp[[i]] <-  as.vector(which(nZ[i,]>thresZ))
            biypv[[i]] <- sz[chop]
            biypn[[i]] <-  sample_namest[chop]

            binn[[i]] <- c(length(cho),length(chon))
            numnn[[i]] <-  as.vector(which(nZ[i,]< -thresZ))
            biynv[[i]] <- sz[chon]
            biynn[[i]] <-  sample_namest[chon]
        } else {
            binn[[i]]    <- c(length(cho),length(chop))
            numnn[[i]]   <-  as.vector(which(nZ[i,]>thresZ))
            biynv[[i]]   <- sz[chop]
            biynn[[i]]   <-  sample_namest[chop]

            binp[[i]]    <- c(length(cho),length(chon))
            numnp[[i]]   <-  as.vector(which(nZ[i,]< -thresZ))
            biypv[[i]]   <- sz[chon]
            biypn[[i]]   <-  sample_namest[chon]

        }





    }

    bic <- cbind(binp,bixv,bixn,biypv,biypn)
    numn <- cbind(numng,numnp)

    bicopp <- cbind(binn,bixv,bixn,biynv,biynn)
    numnopp <- cbind(numng,numnn)




    return(list(bic=bic,numn=numn,bicopp=bicopp,numnopp=numnopp,X=X,np=p))

}





nmfdiv <- function(X,p=5,cyc=100) {


        if (missing(X)) {
            stop("Data matrix X missing. Stopped.")
        }

        if (!is.matrix(X)) {
            X <- as.matrix(X)
        }

        if (!is.numeric(X)) {
            stop("Data matrix X must be numeric. Stopped.")
        }

  # Check that we have non-negative data
if (min(X)<0) {
    stop("Negative values in matrix X. Stopped.")
}

# Dimensions
n <- nrow(X)
l <- ncol(X)

        rownames(X) <- rownames(X, do.NULL = FALSE, prefix = "gene")
        colnames(X) <- colnames(X, do.NULL = FALSE, prefix = "sample")
        rowna <- rownames(X)
        colna <- colnames(X)


       cat("Running NMFDIV on ",n,"x",l," matrix with\n")
       cat("   Number of biclusters ---------------- p: ",p,"\n")
       cat("   Number of maximal iterations ------ cyc: ",cyc,"\n")



# Globally rescale data to avoid potential overflow/underflow
X <- X/max(X)

cent <- as.vector(rep(0.0,n))
scaleData <- as.vector(rep(max(X),n))

# Create initial matrices
L <- matrix (abs(rnorm(n*p)),nrow=n,ncol=p)
Z <- matrix (abs(rnorm(l*p)),nrow=p,ncol=l)
#Z <- Z/(sqrt(rowsum(Z*Z,1:n))*as.vector(rep(1,l)))

# Calculate initial objective
objhistory <- 0.5*sum(sum((X-L%*%Z)^2))


ini <- matrix(1,l,(p+1))
avini <- as.vector(rep(1.0,(p+1)))
xavini <- as.vector(rep(1.0,(l+1)))



# Start iteration
for (i in 1:cyc)
  {

    # Show progress
    print(i)
    print(objhistory)

    # Save old values
    Lold <- L
    Zold <- Z


    # Compute new L and Z (Lee and Seung; NIPS*2000)

    #L <- L*((X/(L%*%Z + 1e-9))%*%t(Z))/rowSums(Z)
    #Z <- Z*(t(L)%*%(X/(L%*%Z + 1e-9)))/colSums(L)

    L <- L*((X/(L%*%Z + 1e-9))%*%t(Z))/tcrossprod(rep(1,n),rowSums(Z))
    Z <- Z*(t(L)%*%(X/(L%*%Z + 1e-9)))/tcrossprod(colSums(L),rep(1,l))



    # Renormalize so rows of Z have constant energy
    norms <- sqrt(rowSums(Z^2))
#    Z <- Z/tcrossprod(norms,rep(1,l))
#    L <- L*tcrossprod(rep(1,n),norms)

    Z <- Z/norms
    L <- L*norms

    # Calculate objective
    newobj <- sum(X*log((X+1e-10)/(L%*%Z)) - X + L%*%Z)
    objhistory <- newobj


  }

        rownames(L) <- rowna
        colnames(L) <- colnames(L, do.NULL = FALSE, prefix = "bicluster")
        clnames <- colnames(L)

        rownames(Z) <- clnames
        colnames(Z) <- colna

        M <- diag(p)
        rownames(M) <- clnames
        colnames(M) <- clnames

        LZ <- L%*%Z

        rownames(LZ) <- rowna
        colnames(LZ) <- colna

        U <- X-LZ

        rownames(U) <- rowna
        colnames(U) <- colna

  return(new('Factorization', parameters=list("nmfdiv",cyc,NULL,NULL,NULL,p,NULL,NULL,NULL,NULL,NULL,NULL,NULL),n=n,p1=p,p2=p,l=l,center=cent,scaleData=scaleData,X=X,L=L,Z=Z,M=M,LZ=LZ,U=U,avini=avini,xavini=xavini,ini=ini,Psi=as.vector(1),lapla=as.matrix(1)))
}


nmfeu <- function(X,p=5,cyc=100) {

        if (missing(X)) {
            stop("Data matrix X missing. Stopped.")
        }

        if (!is.matrix(X)) {
            X <- as.matrix(X)
        }

        if (!is.numeric(X)) {
            stop("Data matrix X must be numeric. Stopped.")
        }


# Check that we have non-negative data
if (min(X)<0) {
    stop("Negative values in matrix X. Stopped.")
}

# Dimensions
n <- nrow(X)
l <- ncol(X)

        rownames(X) <- rownames(X, do.NULL = FALSE, prefix = "gene")
        colnames(X) <- colnames(X, do.NULL = FALSE, prefix = "sample")
        rowna <- rownames(X)
        colna <- colnames(X)


       cat("Running NMFEU on ",n,"x",l," matrix with\n")
       cat("   Number of biclusters ---------------- p: ",p,"\n")
       cat("   Number of maximal iterations ------ cyc: ",cyc,"\n")




# Globally rescale data to avoid potential overflow/underflow
X <- X/max(X)


cent <- as.vector(rep(0.0,n))
scaleData <- as.vector(rep(max(X),n))

ini <- matrix(1,l,(p+1))
avini <- as.vector(rep(1.0,(p+1)))
xavini <- as.vector(rep(1.0,(l+1)))


# Create initial matrices
L <- matrix (abs(rnorm(n*p)),nrow=n,ncol=p)
Z <- matrix (abs(rnorm(l*p)),nrow=p,ncol=l)
#Z <- Z/(sqrt(rowsum(Z*Z,1:n))*as.vector(rep(1,l)))

# Calculate initial objective
objhistory <- 0.5*sum(sum((X-L%*%Z)^2))


# Start iteration
for (i in 1:cyc)
  {

    # Show progress
    print(i)
    print(objhistory)

    # Save old values
    Lold <- L
    Zold <- Z


    # Compute new L and Z (Lee and Seung; NIPS*2000)
    Z <- Z*(t(L)%*%X)/(t(L)%*%L%*%Z + 1e-9)
    L <- L*(X%*%t(Z))/(L%*%Z%*%t(Z) + 1e-9)

    # Renormalize so rows of Z have constant energy
    norms <- sqrt(rowSums(Z^2))
#    Z <- Z/tcrossprod(norms,rep(1,l))
#    L <- L*tcrossprod(rep(1,n),norms)

    Z <- Z/norms
    L <- L*norms



    # Calculate objective
    newobj <- sum(X*log((X+1e-10)/(L%*%Z)) - X + L%*%Z)
    objhistory <- newobj


  }

        rownames(L) <- rowna
        colnames(L) <- colnames(L, do.NULL = FALSE, prefix = "bicluster")
        clnames <- colnames(L)

        rownames(Z) <- clnames
        colnames(Z) <- colna

        M <- diag(p)
        rownames(M) <- clnames
        colnames(M) <- clnames

        LZ <- L%*%Z

        rownames(LZ) <- rowna
        colnames(LZ) <- colna

        U <- X-LZ

        rownames(U) <- rowna
        colnames(U) <- colna

        return(new('Factorization', parameters=list("nmfeu",cyc,NULL,NULL,NULL,p,NULL,NULL,NULL,NULL,NULL,NULL,NULL),n=n,p1=p,p2=p,l=l,center=cent,scaleData=scaleData,X=X,L=L,Z=Z,M=M,LZ=LZ,U=U,avini=avini,xavini=xavini,ini=ini,Psi=as.vector(1),lapla=as.matrix(1)))
}


nprojfunc <- function(s, k1, k2) {


  N <- length(s)


  ones <- as.vector(rep(1,N))



# Start by projecting the point to the sum constraint hyperplane
  v <- s + (k1-sum(s))/N

# Initialize zerocoeff (initially, no elements are assumed zero)

  # zerocoeff <- which(v<=0)
  # v[zerocoeff] <- 0

  zerocoeff <- vector(mode = "integer", length = 0)



  j <- 0
  ende <- 0
  while (ende<1) {

    # This does the proposed projection operator
    midpoint <- ones*k1/(N-length(zerocoeff))
    midpoint[zerocoeff] <- 0
    w <- v-midpoint
    a <- sum(w^2)
    b <- 2*crossprod(w,v)
    c <- sum(v^2)-k2
    t <- b^2-4*a*c
    if (t<0) {t <- 0}
    if (a<1e-10) {a <- 1e-10}

    alphap <- (-b+sqrt(t))/(2*a)
    v <- alphap*w + v

    if ((length(which(v<0))==0)&&(j>1)) {
      ende <- 2
    }
    else {

      j <- j+1

    # Set negs to zero, subtract appropriate amount from rest
      zerocoeff <- which(v<=0)
      v[zerocoeff] <- 0
      tempsum <- sum(v)
      tta <- N - length(zerocoeff)
      if (tta>0)  {
       v <- v + (k1-tempsum)/tta
       }
      v[zerocoeff] <- 0
    }
  }


  return(v)

}




# nmfsc - nonnegative matrix factorization with sparseness constraints
#
# Written by Sepp Hochreiter according to
# Patrik O. Hoyer
# 'Non-negative matrix factorization with sparseness constraints'
# Journal of Machine Learning Research  5:1457-1469, 2004.
#
#
# SYNTAX:
# res <- nmfsc( X, p, sL, sZ,cyc=100)
# res$L
# res$Z
#
# INPUTS:
# X          - data matrix
# p       - number of components (inner dimension of factorization)
# sL         - sparseness of L, in [0,1]
# sZ         - sparseness of Z, in [0,1]
# cyc    - maximal number of iterations
#
# Note: Sparseness is measured on the scale [0,1] where 0 means
# completely distributed and 1 means ultimate sparseness.
#
# NOTE: There is NO CONVERGENCE CRITERION.
#

nmfsc <- function(X,p=5,cyc=100,sL=0.6,sZ=0.6) {

        if (missing(X)) {
            stop("Data matrix X missing. Stopped.")
        }

        if (!is.matrix(X)) {
            X <- as.matrix(X)
        }

        if (!is.numeric(X)) {
            stop("Data matrix X must be numeric. Stopped.")
        }


# Check that we have non-negative data
if (min(X)<0) {
    stop("Negative values in matrix X. Stopped.")
}


# Data dimensions
n <- nrow(X)
l <- ncol(X)

        rownames(X) <- rownames(X, do.NULL = FALSE, prefix = "gene")
        colnames(X) <- colnames(X, do.NULL = FALSE, prefix = "sample")
        rowna <- rownames(X)
        colna <- colnames(X)

       cat("Running NMFSC on ",n,"x",l," matrix with\n")
       cat("   Number of biclusters ---------------- p: ",p,"\n")
       cat("   Number of maximal iterations ------ cyc: ",cyc,"\n")
       cat("   Loading sparseness projection ------ sL: ",sL,"\n")
       cat("   Factor sparseness projection ------- sZ: ",sZ,"\n")


cent <- as.vector(rep(0.0,n))
scaleData <- as.vector(rep(1.0,n))

# Create initial matrices
L <- matrix (rnorm(n*p),nrow=n,ncol=p)
Z <- matrix (rnorm(l*p),nrow=p,ncol=l)


ini <- matrix(1,l,(p+1))
avini <- as.vector(rep(1.0,(p+1)))
xavini <- as.vector(rep(1.0,(l+1)))


# Make initial matrices have correct sparseness
L1a <- sqrt(n)-(sqrt(n)-1)*sL
L1s <- sqrt(l)-(sqrt(l)-1)*sZ

for (i in 1:p)
  {
    L[,i] <- nprojfunc(s=as.vector(L[,i]),k1=L1a,k2=1.0)
    Z[i,] <- nprojfunc(s=as.vector(Z[i,]),k1=L1s,k2=1.0)
  }


# Calculate initial objective
objhistory <- sum((X-L%*%Z)^2)



# Initial stepsizes
stepsizeL <- 1.0
stepsizeZ <- 1.0



# Start iteration
for (i in 1:cyc)
  {


    # Save old values
    Lold <- L
    Zold <- Z

    # ----- Update Z ---------------------------------------


	# Gradient for Z
	dZ <- t(L)%*%(L%*%Z-X)

        begobj <- objhistory

        newobj <- 2.0*begobj




     # Make sure we decrease the objective!
	while (newobj>begobj) {


         # Take step in direction of negative gradient, and project
          Znew <- Z - stepsizeZ*dZ
	# Make initial matrices have correct sparseness
          for (j in 1:p)
            {
              Znew[j,] <- nprojfunc(s=as.vector(Znew[j,]),k1=L1s,k2=1.0)
            }


         # Calculate new objective
	    newobj <- sum((X-L%*%Znew)^2)

	    # If the objective decreased, we can continue...
            stepsizeZ <- stepsizeZ/2.0
            if (stepsizeZ<1e-10) {

                rownames(L) <- rowna
                colnames(L) <- colnames(L, do.NULL = FALSE, prefix = "bicluster")
                clnames <- colnames(L)

                rownames(Z) <- clnames
                colnames(Z) <- colna

                M <- diag(p)
                rownames(M) <- clnames
                colnames(M) <- clnames

                LZ <- L%*%Z

                rownames(LZ) <- rowna
                colnames(LZ) <- colna

                U <- X-LZ

                rownames(U) <- rowna
                colnames(U) <- colna

         return(new('Factorization', parameters=list("nmfsc",cyc,NULL,NULL,NULL,p,sL,sZ,NULL,NULL,NULL,NULL,NULL),n=n,p1=p,p2=p,l=l,center=cent,scaleData=scaleData,X=X,L=L,Z=Z,M=M,LZ=LZ,U=U,avini=avini,xavini=xavini,ini=ini,Psi=as.vector(1),lapla=as.matrix(1)))
            }
        }

	# Slightly increase the stepsize
	stepsizeZ <- stepsizeZ*1.2
	Z <- Znew



    # ----- Update L ---------------------------------------


	# Gradient for L
	dL <- (L%*%Z-X)%*%t(Z)
	begobj <- sum((X-L%*%Z)^2)

        newobj <- 2*begobj

	# Make sure we decrease the objective!
	while (newobj>begobj) {



	    # Take step in direction of negative gradient, and project
	    Lnew <- L - stepsizeL*dL
	    norms <- sqrt(sum(Lnew^2))
            for (j in 1:p)
              {
                Lnew[,j] <- nprojfunc(s=as.vector(Lnew[,j]),k1=L1a,k2=1.0)
              }

	    # Calculate new objective
	    newobj <- sum((X-Lnew%*%Z)^2)

	    stepsizeL <- stepsizeL/2

            if (stepsizeZ<1e-10) {
                rownames(L) <- rowna
                colnames(L) <- colnames(L, do.NULL = FALSE, prefix = "bicluster")
                clnames <- colnames(L)

                rownames(Z) <- clnames
                colnames(Z) <- colna

                M <- diag(p)
                rownames(M) <- clnames
                colnames(M) <- clnames

                LZ <- L%*%Z

                rownames(LZ) <- rowna
                colnames(LZ) <- colna

                U <- X-LZ

                rownames(U) <- rowna
                colnames(U) <- colna

              return(new('Factorization', parameters=list("nmfsc",cyc,NULL,NULL,NULL,p,sL,sZ,NULL,NULL,NULL,NULL,NULL),n=n,p1=p,p2=p,l=l,center=cent,scaleData=scaleData,X=X,L=L,Z=Z,M=M,LZ=LZ,U=U,avini=avini,xavini=xavini,ini=ini,Psi=as.vector(1),lapla=as.matrix(1)))
            }


          }

	# Slightly increase the stepsize
	stepsizeL <- stepsizeL*1.2
	L <- Lnew


    # Calculate objective
    newobj <- 0.5*sum(sum((X-L%*%Z)^2))
    objhistory <- newobj



  }
}

# eqscplot from the package MASS
#
# Plots with Geometrically Equal Scales
#
# From Version: 7.3-3, Date: 2009-10-15
# Maintainer: Brian Ripley <ripley@stats.ox.ac.uk>
#
# To make the package stand alone, this function
# is included as source.
#
#References:
#
#     Venables, W. N. and Ripley, B. D. (2002) _Modern Applied
#     Statistics with S._ Fourth edition.  Springer.

eqscplotLoc <- function (x, y, ratio = 1, tol = 0.04, uin, ...)
{
    dots <- list(...)
    nmdots <- names(dots)
    Call <- match.call()
    Call$ratio <- Call$tol <- Call$uin <- NULL
    if (is.matrix(x)) {
        y <- x[, 2]
        x <- x[, 1]
        if (!is.null(dn <- colnames(x))) {
            xlab0 <- dn[1L]
            ylab0 <- dn[2L]
        }
        else {
            xlab0 <- ""
            ylab0 <- ""
        }
    }
    else if (is.list(x)) {
        y <- x$y
        x <- x$x
        xlab0 <- "x"
        ylab0 <- "y"
    }
    else {
        xlab0 <- deparse(substitute(x))
        ylab0 <- deparse(substitute(y))
    }
    Call$x <- x
    Call$y <- y
    Call$xlab <- if ("xlab" %in% nmdots)
        dots$xlab
    else xlab0
    Call$ylab <- if ("ylab" %in% nmdots)
        dots$ylab
    else ylab0
    xlim <- if ("xlim" %in% nmdots)
        dots$xlim
    else range(x[is.finite(x)])
    ylim <- if ("ylim" %in% nmdots)
        dots$ylim
    else range(y[is.finite(y)])
    midx <- 0.5 * (xlim[2L] + xlim[1L])
    xlim <- midx + (1 + tol) * 0.5 * c(-1, 1) * (xlim[2L] - xlim[1L])
    midy <- 0.5 * (ylim[2L] + ylim[1L])
    ylim <- midy + (1 + tol) * 0.5 * c(-1, 1) * (ylim[2L] - ylim[1L])
    oldpin <- par("pin")
    xuin <- oxuin <- oldpin[1L]/abs(diff(xlim))
    yuin <- oyuin <- oldpin[2L]/abs(diff(ylim))
    if (missing(uin)) {
        if (yuin > xuin * ratio)
            yuin <- xuin * ratio
        else xuin <- yuin/ratio
    }
    else {
        if (length(uin) == 1L)
            uin <- uin * c(1, ratio)
        if (any(c(xuin, yuin) < uin))
            stop("'uin' is too large to fit plot in")
        xuin <- uin[1L]
        yuin <- uin[2L]
    }
    xlim <- midx + oxuin/xuin * c(-1, 1) * diff(xlim) * 0.5
    ylim <- midy + oyuin/yuin * c(-1, 1) * diff(ylim) * 0.5
    Call$xlim <- xlim
    Call$ylim <- ylim
    Call$xaxs <- Call$yaxs <- "i"
    Call[[1L]] <- as.name("plot")
    eval.parent(Call)
}



# The function plot.fabia is based on
# the function plot.mpm in the R package mpm
#
# Version: 1.0-16, Date: 2009-08-26
# Title: Multivariate Projection Methods
# Maintainer: Tobias Verbeke <tobias.verbeke@openanalytics.be>
# Author: Luc Wouters <wouters_luc@telenet.be>


setGeneric("plot")
setMethod("plot", signature(x="Factorization", y="missing"),
 function(x,y,
    Rm=NULL,
    Cm=NULL,
    dim = c(1, 2), # Principal factors to plot.
    zoom = rep(1, 2), # Zoom factor for factor scores and loadings
    col.group = NULL,
    colors = c("orange1", "red", rainbow(length(unique(col.group)), start=2/6, end=4/6)),
    col.areas = TRUE,
    col.symbols = c(1, rep(2, length(unique(col.group)))),
    sampleNames = TRUE,
    rot = rep(-1, length(dim)), # Mirror all axes
    labels = NULL, # character vector of labels (to allow labels to differ from row.names)
    label.tol = 1,
    lab.size = 0.725,
    col.size = 10,
    row.size = 10,
    do.smoothScatter = FALSE, # Plot individual points or density maps
    do.plot = TRUE, # This routine can also be used to calculate the
                    # coordinates, without plotting
                    # legend = FALSE
          ... )
{



  ### error checking and argument matching
  if (missing(x))
    stop ("Argument 'x' (class Factorization) is missing.")

  if (is(x) != "Factorization") {
        stop("Object x is not of the class Factorization. Stopped.")
    }



  if (is.null(Rm)) {
    Rm=rep(1,nrow(x@L))
   }
  if (is.null(Cm)) {
    Cm=rep(1,ncol(x@Z))
  }
  if (is.null(col.group)) {
    col.group = rep(1, ncol(x@Z))
  }


  LI <- x@L
  ZI <- x@Z


  p <- nrow(ZI)

  if (p<2) {
        stop("At least two biclusters are needed for a 2dim plot. Stopped.")
    }

    if (dev.interactive()) {
      x11()
    }

  n <- nrow(LI)
  l <- ncol(ZI)

  X <- x@X
  if (ncol(X)==l) {
      colnames(ZI) <- colnames(X)
  }
  if (nrow(X)==n) {
      rownames(LI) <- rownames(X)

  }


  if (length(x@avini) < p) {
      avini <- rep(1,p)
  } else {
      avini <- x@avini[1:p]
  }

  if (do.smoothScatter && label.tol == 1){
    # if we require all points in the plot to be labelled, no points are left for the density map
    warning("All points selected for labelling, continuing without density map.\n")
    do.smoothScatter <- FALSE
  }
  if(do.smoothScatter && !do.plot){
    warning("Density map plotting requested but plotting not selected. Continuing with plot.\n")
    do.plot <- TRUE # If smoothScatter is set, then also do a plot
  }

  dLL <- 1/(apply(LI,2,function(x) max(abs(x)))+0.001*as.vector(rep(1,p)))
  #  dLL <- 1/sqrt((1/n)* apply(LI,2,function(x) sum(x^2))+0.001*as.vector(rep(1,p)))
  LI <- t(dLL*t(LI))

  dZZ <- 1/(apply(ZI,1,function(x) max(abs(x)))+0.001*as.vector(rep(1,p)))
  #  dZZ <- 1/sqrt( (1/l)*apply(ZI,1,function(x) sum(x^2))+0.001*as.vector(rep(1,p)))
  ZI <- dZZ*ZI



  # labeling of samples on the plot
  if (is.logical(sampleNames)){
    if (length(sampleNames) > 1){
      stop("'sampleNames' should either be a logical of length one or a character of length equal to the number of observations")
    } else {
      sampleLabels <- if (sampleNames) colnames(ZI) else rep("", l)
    }
  } else {
    if (length(sampleNames) != l){
      stop("If 'sampleNames' is a character vector, the length should be equal to the number of observations")
    } else {
      sampleLabels <- sampleNames
    }
  }


  if (is.data.frame(col.group))
    col.group <- unlist(col.group)
  if (length(col.group) != l)
    stop("Length of 'col.group' not equal to number of columns in data.")
  col.group <- as.numeric(as.factor(col.group))


  sud <- sum(avini)
  if (sud>1e-8) {

      soo <- sort(avini, decreasing = TRUE,index.return=TRUE)

      d <- avini[soo$ix]
      U <- LI[,soo$ix]
      V <- t(ZI[soo$ix,])
      contrib <- d/sud
  } else {
      U <- LI
      V <- t(ZI)
      contrib <- rep((1/p),p)
  }

  #
  # Scaling, scores and loadings
  #
  ### scaling: alpha and beta values for different scaling options
  # Calculate weighted factor scores (for rows)
  Z <- V[, dim]

  # Calculate weighted factor loadings (for columns)
  L <- U[,dim]

  # Rotation
  # Default: flips both the X and Y axes
  Z <- Z * matrix(rot, ncol = ncol(Z), nrow = nrow(Z), byrow = TRUE)
  L <- L * matrix(rot, ncol = ncol(L), nrow = nrow(L), byrow = TRUE)

  #
  # zooming
  zz <- Z * zoom[1]
  ll <- L * zoom[2]


  #
  # Calculate which rows are far from the origin (most specific)
  #

  iz <- rep(TRUE, nrow(Z))

  il <- rep(TRUE, nrow(L))

  if (length(dim) == 2){
      DL <- L[, 1]^2 + L[, 2]^2  # distance from center
  # Threshold for labels
      if (label.tol > 1) {# label.tol most distant rows are plotted as circles and labelled
          thres <- sort(DL, decreasing = TRUE)[min(length(DL), floor(label.tol))]
      }
      else {# label.tol percent most distant rows are plotted as circles and labelled
          thres <- if (label.tol == 0) Inf else quantile(DL, probs = 1-label.tol)}

  #
  # Positioning
  #
      iselt <- il & (DL >= thres)
  } else {


      DL <- L[, 1]^2 + L[, 2]^2 + L[, 3]^2  # distance from center
  # Threshold for labels
      if (label.tol > 1) {# label.tol most distant rows are plotted as circles and labelled
          thres <- sort(DL, decreasing = TRUE)[min(length(DL), floor(label.tol))]
      }
      else {# label.tol percent most distant rows are plotted as circles and labelled
          thres <- if (label.tol == 0) Inf else quantile(DL, probs = 1-label.tol)
      }

  #
  # Positioning
  #
      iselt <- il & (DL >= thres)

  }


  isel <- which(iselt)
  # Only draw a plot if the projection is 2D and the user requested a plot (default)
 #
  if (do.plot && (length(dim) == 2)){

   # Compute range of plot
    #

   xrange <- range(ll[, 1], zz[, 1], 0)
    yrange <- range(ll[, 2], zz[, 2], 0)
    xrange <- ifelse(rep(diff(xrange) == 0, 2),
        c(-1, 1), # Default range [-1,1] if calculated range is 0
        xrange + diff(xrange) * c(-0.1, 0.1)) # Else expand range by 10%
    yrange <- ifelse(rep(diff(yrange) == 0, 2),
        c(-1, 1),
        yrange + diff(yrange) * c(-0.1, 0.1))
    # the eqscplot funtion (ratio=1) takes care of equally scaled axes, so make sure
    # we have the correct range for each axis here.

    #
    # Clipping
    # (replace coordinates outside the plotted range by the min or max coordinates)
    zz[,1] <- pmin(pmax(zz[,1], xrange[1]), xrange[2])
    zz[,2] <- pmin(pmax(zz[,2], yrange[1]), yrange[2])
    ll[,1] <- pmin(pmax(ll[,1], xrange[1]), xrange[2])
    ll[,2] <- pmin(pmax(ll[,2], yrange[1]), yrange[2])

   #
    # Set-up plot
    #
    opar <- par(pty = "m") # preserve configuration
    # Create a window with the maximal plotting region
    # Equal scale plot function from MASS library
    dotList <- list(...)
    if (is.null(dotList$sub)){
      dotList$sub <- paste("FABIA", sep="")
      if (is.null(dotList$cex.sub))
        dotList$cex.sub <- 0.85
    }
    dotList$cex.sub <- if (is.null(dotList$cex.sub)) 0.85 else dotList$cex.sub
    if (is.null(dotList$xlab))
      dotList$xlab <- paste("BC", dim[1], ": ", 100 * round(contrib[dim[1]], 2), "%,  ", floor(d[dim[1]]),sep = "")
    if (is.null(dotList$ylab))
      dotList$ylab <- paste("BC", dim[2], ": ", 100 * round(contrib[dim[2]], 2), "%,  ", floor(d[dim[2]]), sep = "")

    dotList$x <- xrange
    dotList$y <- yrange
    dotList$ratio <- 1
    dotList$tol <- 0
    dotList$type <- "n"
    dotList$axes <- FALSE
    dotList$cex.lab = 0.85


    do.call("eqscplotLoc", dotList)


    #
    # Scales
    usr <- par("usr") # Retrieve extremes of coordinates in the plotting region
    sx <- diff(usr[c(1,2)]) / (25.4 * par("pin")[1, drop = TRUE]) # scale in mm
    sy <- diff(usr[c(3,4)]) / (25.4 * par("pin")[2, drop = TRUE])

  #
    # Plot rows close to 0,0 as unlabelled dots or as density maps
    #
    # Select rows to be plotted
    idlt <- il & (DL < thres)

    idl <- which(idlt)

    if (!do.smoothScatter) # plot inner points as unlabelled dots
      points(ll[idl,1], ll[idl,2], col = colors[1], cex = 0.825, lwd = 2)
    else # plot as density maps
      smoothScatter(x = ll[idl,1], y = ll[idl,2], nbin = 256, nrpoints = 0,
          add = TRUE, colramp = colorRampPalette(c("white", "burlywood"))) # add image to current eqscplot axes, instead of overwritting
    # Alternative to adding to the axes set up by eqscplot, use the following
    # parameters to set up smoothScatter's own axes.
    # xlim = xrange, ylim = yrange, # make sure we can add distant points later
    # asp=1, # equal scale axes
    # xaxt = "n", yaxt = "n", # do not plot the axes scales
    # xlab = paste("PC", dim[1], " ", 100 * round(x$contrib[dim[1]], 2), "%", sep = ""),
    # ylab = paste("PC", dim[2], " ", 100 * round(x$contrib[dim[2]], 2), "%", sep = ""),


    ### plot distant rows as circles with areas proportional to Rm
    sqs <- 0.5 * sx * pmax(0.02, row.size * sqrt((Rm) / (max(Rm))))
    yoffset <- sy * (2 + sqs / sx)






    if (length(isel) > 0){ # if there is at least 1 point to plot
      symbols(ll[isel, 1], ll[isel, 2], circle = sqs[isel],
        inches = FALSE, lwd = 3, add = TRUE, fg = colors[2])
      if (is.null(labels)) labels <- rownames(LI)
      text(ll[isel, 1], ll[isel, 2] - yoffset[isel], adj = c(0.5, 1),
        cex = lab.size, labels = labels[isel], # x$row.names[isel]
        col = colors[2])
    }

    ### plot columns with indication of column-grouping
    iGroup <- unique(col.group[iz]) # unique groups in columns to be plotted
    for (i in 1:length(iGroup)){
      ii <- iz & (col.group == iGroup[i]) # Select columns in group i selected for plotting
      if (col.areas)
      { # use squares with size (ie. area) proportional to x$Cm
        sqs <- 0.5 * sx * pmax(0.02, col.size * sqrt((Cm[ii]) / (max(Cm))))
        yoffset <- sy * (5 + sqs / (2 * sx))
        symbols(zz[ii, 1], zz[ii, 2],
            square = sqs, inches = FALSE, lwd = 3, add = TRUE, fg = colors[2+iGroup[i]])
        # if (sampleNames){
          text(zz[ii,1], zz[ii,2] + yoffset,
              adj=c(0.5, 1), cex=lab.size, labels=sampleLabels[ii], col=colors[2+iGroup[i]]) # x$col.names[ii]
        # }
      }
      else # Use different symbols, ignore size
      {
        yoffset <- sy * (5 + col.size / 5)
        points(zz[ii, 1], zz[ii, 2],
            pch = col.symbols[iGroup[i]], col = colors[2+iGroup[i]],
            cex = col.size / (25.4 * par("csi")), lwd = 3)
        text(zz[ii, 1], zz[ii, 2] + yoffset,
            adj = c(0.5, 1), cex = lab.size, labels = sampleLabels[ii], col = colors[2+iGroup[i]]) # x$col.names[ii]
      }
    }

    ### finish plot, put crozz on 0,0, box, and legend
    lines(c(-2.5,2.5) * sx, c(0,0), lwd=3)
    lines(c(0,0), c(-2.5,2.5) * sy, lwd=3)
    box()

    # legend
#    legendArg <- match.arg(legend)
#    if (legendArg != "none"){
#      legend(legendArg,
#        legend = levels(pData(ALL)$BT),
#        text.col = col.group, bty='n')
#    }

    par(opar) # Restore plotting configuration
}



                                        #
  # Return value: list of coordinates and indication of most distant (most specific) points
  #

  if (length(dim) == 1){
      rows <- cbind(ll, iselt)
      dimnames(rows) <- list(rownames(LI), c("X", "Select"))
    dimnames(zz) <- list(colnames(ZI), c("X"))
  } else if (length(dim) == 2){
      rows <- cbind(ll, iselt)
      dimnames(rows) <- list(rownames(LI), c("X", "Y", "Select"))
      dimnames(zz) <- list(colnames(ZI), c("X", "Y"))
  } else if (length(dim) == 3){
    dimnames(rows) <- list(rownames(LI), c(paste("Prf", dim, sep=""), "Select"))
    dimnames(zz) <- list(colnames(ZI), paste("Pcf", dim, sep=""))
  } else { # More than 3 dimensions requested
    dimnames(rows) <- list(rownames(LI), c(paste("Prf", dim, sep=""), "Select"))
    dimnames(zz) <- list(colnames(ZI), paste("Pcf", dim, sep=""))
  }
  r <- list(Rows = rows, Columns = zz)
})







