\name{projfunc}
\alias{projfunc}
\title{Projection of a Vector to a Sparse Vector}
\description{

  \code{projfunc}: \R implementation of  \code{projfunc}.
}
\usage{

projfunc(s, k1, k2)

}
\arguments{
  \item{s}{data vector.}
  \item{k1}{sparseness, l1 norm constraint.}
  \item{k2}{l2 norm constraint.}
}
\details{

  The projection is done according to Hoyer, 2004: given an
  \eqn{l_1}-norm and an \eqn{l_2}-norm minimize the Euclidean distance
  to the original vector.
  The projection is a convex quadratic problem which is solved
  iteratively where at each iteration at least one component is set to
  zero.

  In the applications,
  instead of the  \eqn{l_1}-norm a sparseness measurement is used
  which relates the \eqn{l_1}-norm to the \eqn{l_2}-norm.


  Implementation in \R.

}
\value{
  \item{v}{sparse projected vector.}

    }
\seealso{
\code{\link{fabi}},
\code{\link{fabia}},
\code{\link{fabiap}},
\code{\link{fabias}},
\code{\link{fabiasp}},
\code{\link{mfsc}},
\code{\link{nmfdiv}},
\code{\link{nmfeu}},
\code{\link{plot}},
\code{\link{extract_plot}},
\code{\link{extract_bic}},
\code{\link{plotBicluster}},
\code{\link{Factorization}},
\code{\link{nprojfunc}},
\code{\link{projfunc}},
\code{\link{estimateMode}},
\code{\link{make_fabi_data}},
\code{\link{make_fabi_data_blocks}},
\code{\link{make_fabi_data_pos}},
\code{\link{make_fabi_data_blocks_pos}},
\code{\link{myImagePlot}},
\code{\link{Breast_A}},
\code{\link{DLBCL_B}},
\code{\link{Multi_A}},
\code{\link{eqscplotLoc}},
\code{\link{summary}},
\code{\link{show}},
\code{\link{showSelected}},
\code{\link{fabiaDemo}},
\code{\link{fabiaVersion}}
}
\author{Sepp Hochreiter}
\examples{

#---------------
# DEMO
#---------------

size <- 30
sparseness <- 0.7

s <- as.vector(rnorm(size))
sp <- sqrt(1.0*size)-(sqrt(1.0*size)-1.0)*sparseness

ss <- projfunc(s,k1=sp,k2=1)

s
ss

}
\references{
  Patrik O. Hoyer,
  \sQuote{Non-negative Matrix Factorization with Sparseness Constraints},
  Journal of Machine Learning Research 5:1457-1469, 2004.
}
\keyword{methods}
\concept{sparse coding}
\concept{sparse matrix factorization}
